 /* Last changed Time-stamp: <2007-10-30 14:06:22 htafer> */
/*                
	   compute the duplex structure of two RNA strands,
		allowing only inter-strand base pairs.
	 see cofold() for computing hybrid structures without
			     restriction.
			     Ivo Hofacker
			  Vienna RNA package

*/


/*
  library containing the function used in rnaplex
  the program rnaplex uses the following function
  Lduplexfold: finds high scoring segments
  it stores the end-position of these segments in an array 
  and call then for each of these positions the duplexfold function 
  which allows to make backtracking for each of the high scoring position
  It allows to find suboptimal partially overlapping (depends on a a parameter) 
  duplexes between a long RNA and a shorter one.
  Contrarly to RNAduplex, the energy model is not in E~log(N), 
  where N is the length of an interial loop but used an affine model, 
  where the extension and begin parameter are fitted to the energy
  parameter used by RNAduplex. This allows to check for duplex between a short RNA(20nt) 
  and a long one at the speed of 1Mnt/s. At this speed the whole genome (3Gnt) can be analyzed for one siRNA 
  in about 50 minutes.
  The algorithm is based on an idea by Durbin and Eddy:when the alginment reach a value larger than a
  given threshold this value is stored in an array. When the alignment score goes 
  then under this threshold, the alignemnent begin from this value, in that way the backtracking allow us 
  to find all non-overlapping high-scoring segments. 
  For more information check "durbin, biological sequence analysis"
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "utils.h"
#include "energy_par.h"
#include "fold_vars.h"
#include "fold.h"
#include "pair_mat.h"
#include "params.h"
#include "plex.h"
#include "ali_plex.h"
/*@unused@*/
static char rcsid[] UNUSED = "$Id: plex.c,v 1.14 2007/06/12 12:50:16 htafer Exp $";
//int subopt_sorted=0;

#define PUBLIC
#define PRIVATE static

#define STACK_BULGE1  1   /* stacking energies for bulges of size 1 */
#define NEW_NINIO     1   /* new asymetry penalty */
#define ARRAY 32          /*array size*/
#define UNIT 100
#define MINPSCORE -2 * UNIT
PRIVATE void  encode_seqs(const char *s1, const char *s2);
PRIVATE short *encode_seq(const char *seq);
//PRIVATE void  my_encode_seq(const char *s1, const char *s2);
PRIVATE void  update_dfold_params(void);
//PRIVATE int   compare(const void *sub1, const void *sub2);
//PRIVATE int   compare_XS(const void *sub1, const void *sub2);
//PRIVATE duplexT* backtrack(int threshold, const int extension_cost);
//static void  print_struct(duplexT const *dup);

//PRIVATE int   print_struct(duplexT const *dup);
//PRIVATE int   get_rescaled_energy(duplexT const *dup);

PRIVATE char * backtrack_C(int i, int j, const int extension_cost, const char * structure, int *E);
PRIVATE void   find_max_C(const int *position, const int *position_j, const int delta, const int threshold, const int constthreshold, const int length, const char *s1, const char *s2, const int extension_cost, const int fast, const char* structure);
PRIVATE void   plot_max_C(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast, const char* structure);


PRIVATE char *   backtrack_CXS(int i, int j, const int** access_s1, const int** access_s2, const char* structure, int *E);
PRIVATE void find_max_CXS(const int *position, const int *position_j,const int delta, const int threshold, const int constthreshold, const int alignment_length, const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast,const char* structure);
PRIVATE void     plot_max_CXS(const int max, const int max_pos, const int max_pos_j, const int alignment_length,const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast, const char* structure);
PRIVATE duplexT duplexfold_C(const char *s1, const char *s2, const int extension_cost, const char* structure);
PRIVATE duplexT duplexfold_CXS(const char *s1, const char *s2,const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int threshold ,const char* structure);


/*@unused@*/

#define MAXSECTORS      500     /* dimension for a backtrack array */
#define LOCALITY        0.      /* locality parameter for base-pairs */

#define MIN2(A, B)      ((A) < (B) ? (A) : (B))
#define MAX2(A, B)      ((A) > (B) ? (A) : (B))

PRIVATE paramT *P = NULL;
PRIVATE int   **c;//, **in, **bx, **by;      /* energy array used in duplexfold */
//PRIVATE int ****c_XS;
PRIVATE int  **lc, **lin, **lbx, **lby;   /* energy array used in Lduplexfold
					     this arrays contains only 3 columns
					     In this way I reduce my memory use and
					     I can make most of my computation and 
					     accession in the computer cash
					     which is the main performance boost*/
					     


/*PRIVATE int last_cell;                    this variable is the last_cell containing
					    the information about the alignment
					    useful only if there is an alignment 
					    which extends till the last nucleotide of 
					    the long sequence*/

PRIVATE short  *S1, *SS1, *S2, *SS2;/*contains the sequences*/
PRIVATE int   n1,n2;    /* sequence lengths */
PRIVATE int n3, n4; /*sequence length for the duplex*/;
extern  int  LoopEnergy(int n1, int n2, int type, int type_2,
                        int si1, int sj1, int sp1, int sq1);
PRIVATE int delay_free=0;


/*-----------------------------------------------------------------------duplexfold_XS---------------------------------------------------------------------------*/

PRIVATE duplexT duplexfold_CXS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, 
			       const int i_pos, const int j_pos, const int threshold, const char* structure) {
  int i, j,p,q,Emin=INF, l_min=0, k_min=0;
  char *struc;
  struc=NULL;
  duplexT mfe;
  int bonus=-10000;
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);

  int *previous_const;
  previous_const=(int *) space(sizeof(int) * (n4+1));
  j=0;
  previous_const[j]=1;
  int prev_temp = 1;
  while(j++<n4){
    if(structure[j-1]=='|'){
      previous_const[j]=prev_temp;
      prev_temp=j;
    }
    else{
      previous_const[j]=prev_temp;
    }
  }
  if ((!P) || (fabs(P->temperature - temperature)>1e-6)) {
    update_fold_params();  P = scale_parameters();
    make_pair_matrix();
  }
  
  c = (int **) space(sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++) c[i] = (int *) space(sizeof(int) * (n4+1));
  for (i=0; i<=n3; i++){
    for(j=0;j<=n4;j++){
      c[i][j]=INF;
    }
  }
  encode_seqs(s1, s2);
  int type, type2, type3, E, k,l;
  i=n3-1; j=2;
  type = pair[S1[i]][S2[j]]; 
  if(!type){
    printf("Error during initialization of the duplex in duplexfold_XS\n");
    mfe.structure=NULL;
    mfe.energy = INF;
    return mfe;
  }
  c[i][j] = P->DuplexInit + (structure[j-1]=='|' ? bonus : 0 ); //check if first pair is constrained 
  if (type>2) c[i][j] += P->TerminalAU;
  c[i][j]+=P->dangle3[rtype[type]][SS1[i+1]];
  c[i][j]+=(!(structure[j-2] == '|' ) ? P->dangle5[rtype[type]][SS2[j-1]] : 0);
  for (k=i-1; k>0 ; k--) {
    c[k+1][0]=INF;
    for (l=j+1; l<=n4; l++) {
      c[k][l]=INF;
      int bonus_2 = (structure[l-1]=='|'? bonus : 0 ); //check if position is constrained and prepare bonus accordingly
      type2 = pair[S1[k]][S2[l]];
      if (!type2) continue;
      for (p=k+1; p< n3 && p<k+MAXLOOP-1; p++){
	for (q = l-1; q >= previous_const[l] && q > 1; q--) {
	  if (p-k+l-q-2>MAXLOOP) break;
	  type3=pair[S1[p]][S2[q]];
	  if(!type3) continue;
	  E = LoopEnergy(p-k-1, l-q-1, type2, rtype[type3],SS1[k+1], SS2[l-1], SS1[p-1], SS2[q+1]) + bonus_2;
	  c[k][l] = MIN2(c[k][l], c[p][q]+E);
	}
      }
      E = c[k][l]; 
      if (type2>2) E += P->TerminalAU;
      E+=access_s1[i-k+1][i_pos]+access_s2[l-1][j_pos+(l-1)-1];
      if (k>1) E += P->dangle5[type2][SS1[k-1]]; 
      if (l<n4 && !(structure[l]=='|')) E += P->dangle3[type2][SS2[l+1]];
      if (E<Emin) {
	Emin=E; k_min=k; l_min=l;
      } 
    }
  }
  //  mfe.energy =((float)  c[k_min][l_min])/100; //mfe.energy without dangle                                                                                                        
  //  mfe.energy=Emin;
  //type2=pair[S1[k_min]][S2[l_min]];
  //if (type2>2) Emin  += P->TerminalAU;
  free(previous_const);
  if(Emin  > threshold){
    mfe.energy=INF;
    mfe.ddG=INF;
    mfe.structure=NULL;
    for (i=0; i<=n3; i++) free(c[i]);
    free(c);
    free(S1); free(S2); free(SS1); free(SS2);
    return mfe;
  } else{
    struc = backtrack_CXS(k_min, l_min, access_s1, access_s2,structure,&Emin);
  }


//  type2=pair[S1[k_min]][S2[l_min]];
//  if (type2>2) Emin  += P->TerminalAU;
//  
//  if(Emin > threshold || mfe.energy >0){ 
//    mfe.structure=NULL; 
//    for (i=0; i<=n3; i++) free(c[i]);
//    free(c);
//    free(S1); free(S2); free(SS1); free(SS2);
//    return mfe;
//  } else{ 
//    struc = backtrack_XS(k_min, l_min, access_s1, access_s2);
//  }
  //lets take care of the dangles
  //find best combination 
  int dx_5, dx_3, dy_5, dy_3,dGx,dGy,bonus_x;
  dx_5=0; dx_3=0; dy_5=0; dy_3=0;dGx=0;dGy=0;bonus_x=0;
  //x--------x
  // ||||||||
  //x--------x
   dGx = access_s1[i-k_min+1][i_pos];dx_3=0; dx_5=0;bonus_x=0; 
/*   if(dGx + bonus_x > P->dangle3[rtype[type]][SS1[i+1]] + access_s1[i-k_min+2][i_pos+1]){ */
/*     dGx =  access_s1[i-k_min+2][i_pos+1]; */
/*     bonus_x = P->dangle3[rtype[type]][SS1[i+1]]; */
/*     dx_3=1; */
/*     dx_5=0; */
/*   } */
/*   if(dGx  > P->dangle5[type2][SS1[k_min-1]] + access_s1[i-k_min+2][i_pos]) { */
/*     dx_5=1; */
/*     dx_3=0; */
/*     bonus_x = P->dangle5[type2][SS1[k_min-1]]; */
/*     dGx = access_s1[i-k_min+2][i_pos]; */
/*   } */
/*   if(dGx > P->dangle5[type2][SS1[k_min-1]] + P->dangle3[rtype[type]][SS1[i+1]] + access_s1[i-k_min+3][i_pos+1]){ */
/*     dx_5=dx_3=1; */
/*     bonus_x = P->dangle5[type2][SS1[k_min-1]] + P->dangle3[rtype[type]][SS1[i+1]] ; */
/*     dGx = access_s1[(n3-2)-k_min+3][i_pos+1]; */
/*   } */
   dGy = access_s2[l_min-j+1][j_pos + (l_min-1)-1]; 
/*   bonus_y = dy_5 = dy_3 = 0; */
/*   if(dGy + bonus_y > P->dangle5[rtype[type]][SS2[(l_min-1)-1]] +  */
/*      access_s2[l_min-1+1][j_pos  + (l_min -1) -1  ]){ */
/*     dGy = access_s2[l_min-1+1][j_pos -1 + (l_min -1) -1]; */
/*     bonus_y = P-> dangle5[rtype[type]][SS2[(l_min-1)-1]]; */
/*     dy_3 = 0; */
/*     dy_5 = 1; */
/*   } */
/*   if(dGy + bonus_y > P->dangle3[type2][SS2[l_min+1]] + access_s2[l_min-1+1][j_pos + 1 + (l_min-1) -1]){ */
/*     dGy = access_s2[l_min-1+1][j_pos +1 +(l_min-1) -1]; */
/*     bonus_y = P->dangle3[type2][SS2[l_min+1]]; */
/*     dy_3=0; */
/*     dy_5=1; */
/*   } */
/*   if(dGy+bonus_y > P->dangle3[type2][SS2[l_min+1]] + P->dangle5[rtype[type]][SS2[(l_min-1)-1]] + */
/*      access_s2[l_min-1+2][j_pos  + 1 + (l_min -1) -1]){ */
/*     dGy = access_s2[l_min-1+2][j_pos  + 1 + l_min -1 -1]; */
/*     bonus_y = P->dangle3[type2][SS2[l_min+1]] + P-> dangle5[rtype[type]][SS2[(l_min-1)-1]]; */
/*     dy_3=1; */
/*     dy_5=1; */
/*   } */
  mfe.tb=i_pos -9 - i + k_min -1 -dx_5;
  mfe.te=i_pos -9 -1 + dx_3;
  mfe.qb=j_pos -9 -1 - dy_5;
  mfe.qe=j_pos + l_min -3 -9 + dy_3;
  mfe.ddG=(double) Emin * 0.01;
  mfe.dG1=(double) dGx*0.01 ;
  mfe.dG2=(double) dGy*0.01 ; 
  //mfe.energy += bonus_y + bonus_x;
  mfe.energy= mfe.ddG - mfe.dG1 - mfe.dG2;

  mfe.structure = struc;
  for (i=0; i<=n3; i++) free(c[i]);
  free(c);
  free(S1); free(S2); free(SS1); free(SS2);
  return mfe;
}


PRIVATE char *backtrack_CXS (int i, int j, const int **access_s1,const int **access_s2,const char* structure, int *Emin ) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0;
  char *st1, *st2, *struc;
  int *previous_const;
  int bonus=-10000;
  previous_const=(int *) space(sizeof(int) * (n4+1));
  int j_temp=0;
  previous_const[j_temp]=1;
  int prev_temp = 1;
  while(j_temp++<n4){
    if(structure[j_temp-1]=='|'){
      previous_const[j_temp]=prev_temp;
      prev_temp=j_temp;
    }
    else{
      previous_const[j_temp]=prev_temp;
    }
  }
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  i0=i;/*MAX2(i-1,1);*/j0=j;/*MIN2(j+1,n4);*/
  while (i<=n3-1 && j>=2) {
    int bonus_2 = (structure[j-1]== '|'? bonus: 0);
    E = c[i][j]; traced=0;
    st1[i-1] = '(';
    st2[j-1] = ')'; 
    type = pair[S1[i]][S2[j]];
    if (!type) nrerror("backtrack failed in fold duplex bli");
    for (k=i+1; k<=n3 && k>i-MAXLOOP-2; k++) {
      for (l=j-1; l >= previous_const[j] && l>=1; l--) {
	int LE;
	if (i-k+l-j-2>MAXLOOP) break;
	type2 = pair[S1[k]][S2[l]];
	if (!type2) continue;
	LE = LoopEnergy(k-i-1, j-l-1, type, rtype[type2], SS1[i+1], SS2[j-1], SS1[k-1], SS2[l+1]) + bonus_2;
	if (E == c[k][l]+LE) {
	  *Emin-=bonus_2;
	  traced=1; 
	  i=k; j=l;
	  break;
	}
      }
      if (traced) break;
    }
    if (!traced) { 
      if (i<n3) E -= P->dangle3[rtype[type]][SS1[i+1]];//+access_s1[1][i+1];
      if (j>1)  E -= (!(structure[j-2]=='|') ? P->dangle5[rtype[type]][SS2[j-1]] : 0);//+access_s2[1][j+1];
      if (type>2) E -= P->TerminalAU;
      //break;
      if (E != P->DuplexInit + bonus_2)  {
        nrerror("backtrack failed in fold duplex bal");
      } else {
	*Emin-=bonus_2;
	break;
      }
    }
  }
  //if (i<n3)  i++;
  //if (j>1)   j--;
  struc = (char *) space(i-i0+1+j0-j+1+2);
  for (k=MAX2(i0,1); k<=i; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j; k<=j0; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i0-1,0)); strcat(struc, "&"); 
  strcat(struc, st2+j-1);
  free(st1); free(st2);free(previous_const);
  return struc;
}


duplexT** Lduplexfold_CXS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int threshold, const int alignment_length, const int delta, const int fast, const char* structure,const int il_a, const int il_b, const int b_a, const int b_b)//, const int target_dead, const int query_dead)
{
  
  int i, j;
  int bopen=b_b;
  int bext=b_a;
  int iopen=il_b;
  int iext_s=2*il_a;//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  //int temp;
  int min_j_colonne;
  int max=INF;
  int bonus=-10000;
  int constthreshold=0; //minimal threshold corresponding to a structure complying to all constraints
  i=0;
  while(structure[i]!='\0'){
    if(structure[i]=='|') constthreshold+=bonus;
    i++;
  }    
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1
  //int nsubopt=10;  //total number of subopt
  int *position; //contains the position of the hits with energy > E
  int *position_j;
  //int const5end; //position of the 5'most constraint. Only interaction reaching this position are taken into account.
  //const5end = strchr(structure,'|') - structure;
  //const5end++;
  n1 = (int) strlen(s1);
  n2 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  position = (int *) space ((delta+n1+3+delta) * sizeof(int));
  position_j= (int *) space ((delta+n1+3+delta) * sizeof(int));
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
    update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  lc = (int **) space(sizeof(int *) * 4);
  lc[0] = (int *) space(sizeof(int) * (n2+5)); 
  lc[1] = (int *) space(sizeof(int) * (n2+5));
  lc[2] = (int *) space(sizeof(int) * (n2+5));
  lc[3] = (int *) space(sizeof(int) * (n2+5));
  
  /*   local int array initialisation-------------------------------------------*/
  lin = (int**) space (sizeof(int *) * 4);
  lin[0] = (int *) space(sizeof(int) * (n2+5));
  lin[1] = (int *) space(sizeof(int) * (n2+5));
  lin[2] = (int *) space(sizeof(int) * (n2+5));
  lin[3] = (int *) space(sizeof(int) * (n2+5));
  
  /*   local bx array initialization--------------------------------------------*/ 
  lbx = (int**) space (sizeof(int *) * 4);
  lbx[0] = (int *) space(sizeof(int) * (n2+5));
  lbx[1] = (int *) space(sizeof(int) * (n2+5));
  lbx[2] = (int *) space(sizeof(int) * (n2+5));
  lbx[3] = (int *) space(sizeof(int) * (n2+5));
  
  /*   local by array initialization--------------------------------------------*/ 
  lby = (int**) space (sizeof(int *) * 4);
  lby[0] = (int *) space(sizeof(int) * (n2+5));
  lby[1] = (int *) space(sizeof(int) * (n2+5));
  lby[2] = (int *) space(sizeof(int) * (n2+5));
  lby[3] = (int *) space(sizeof(int) * (n2+5));
  
  int di1,di2,di3,di4; //contains accessibility penalty
  
  
  
  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(j=n2+4;j>=0;j--) {
    /*  --------------------------------------------------------local array*/
    lbx[0][j]=lbx[1][j]=lbx[2][j]=lbx[3][j]=INF;//no bulge before 1
    lin[0][j]=lin[1][j]=lin[2][j]=lin[3][j]=INF;//no in before  1
    lc[0][j] =lc[1][j] =lc[2][j] = lc[3][j]=INF; //no bulge and no in before n2
    lby[0][j]=lby[1][j]=lby[2][j]=lby[3][j]=INF;
  }
  /*--------------------------------------------------------local array*/
  
  
  //-------------------------------------------------------------matrix initialisierung
  i=10 /*target_dead*/; //start from 2 (	i=4) because no structure allowed to begin with a single base pair
  i_length= n1 - 9  /*- target_dead*/ ; 
  while(i < i_length) {
    int idx=i%4;
    int idx_1=(i-1)%4;
    int idx_2=(i-2)%4;
    int idx_3=(i-3)%4;
    di1 = access_s1[5][i]   - access_s1[4][i-1];	   
    di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;
    di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2;
    di4 = access_s1[5][i-3] - access_s1[4][i-4] + di3;
    di1=MIN2(di1,165);
    di2=MIN2(di2,330);
    di3=MIN2(di3,495);
    j=n2 - 9 /*- query_dead*/; //start from n2-1 because no structure allow to begin with a single base pair 
    while (--j > 9/*query_dead - 1*/) {
      //----------------------------------------------------------update lin lbx lby matrix
      int bonus_2 = (structure[j-1]=='|' ? bonus :0 );
      int dj1,dj2,dj3,dj4;
      dj1 = access_s2[5][j+4] - access_s2[4][j+4];	
      dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1;
      dj3 = access_s2[5][j+6] - access_s2[4][j+6] + dj2;
      dj4 = access_s2[5][j+7] - access_s2[4][j+7] + dj3;
      dj1=MIN2(dj1,165);
      dj2=MIN2(dj2,330);
      dj3=MIN2(dj3,495);
      int type2, type,temp;
      type  = pair[S1[i]][S2[j]];
      lc[idx][j]= type ? P->DuplexInit + access_s1[1][i] + access_s2[1][j] + bonus_2 : INF;
      if(!bonus_2){
	type2=pair[S2[j+1]][S1[i-1]];
	lin[idx][j]=MIN2(lc[idx_1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+di1+dj1+iopen+iext_s,lin[idx_1][j]+iext_ass + di1); 
	lin[idx][j]=MIN2(lin[idx][j],lin[idx][j+1]+iext_ass + dj1); //Iij=ci-1j+1+M+iopen+2iext
	lin[idx][j]=MIN2(lin[idx][j],lin[idx_1][j+1]+iext_s + di1 + dj1);
	type2=pair[S2[j+1]][S1[i]];
	lby[idx][j]=MIN2(lby[idx][j+1]+bext + dj1 , lc[idx][j+1]+bopen+bext+(type2>2?P->TerminalAU:0)+ dj1);
      }
      else{
	lin[idx][j] = lby[idx][j] = INF; //all loop containing "|" are rejected
      }
      type2=pair[S2[j]][S1[i-1]];
      lbx[idx][j]=MIN2(lbx[idx_1][j]+bext + di1, lc[idx_1][j]+bopen+bext+(type2>2?P->TerminalAU:0) + di1);
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;} 
      lc[idx][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+(structure[j]=='|'? 0 : P->dangle3[type][SS2[j+1]]);
      //type > 2 -> no GC or CG pair
      //------------------------------------------------------------------update c  matrix 
      // Be careful, no lc may come from a region where a "|" is in a loop, avoided in lin = lby = INF ... jedoch fuer klein loops muss man aufpassen ..
      type2=pair[S1[i-1]][S2[j+1]]; 
      lc[idx][j]=MIN2(lc[idx_1][j+1]+P->stack[type2][rtype[type]]+di1+dj1, lc[idx][j]); //0x0+1x1
      type2=pair[S1[i-2]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1,lc[idx][j]);//0x1 +1x1
      //kleine loops checks wird in den folgenden if test gemacht.
      if(!(structure[j]=='|')){
	type2=pair[S1[i-1]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2,lc[idx][j]);//1x0 + 1x1
	type2=pair[S1[i-2]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2, lc[idx][j]); // 1x1 +1x1
	type2 = pair[S1[i-3]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2, lc[idx][j]); // 2x1 +1x1
	if(!(structure[j+1]=='|')){
	  type2 = pair[S1[i-3]][S2[j+3]];
	  lc[idx][j]=MIN2(lc[idx_3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3,lc[idx][j]);//2x2 + 1x1
	  type2 = pair[S1[i-2]][S2[j+3]];
	  lc[idx][j]=MIN2(lc[idx_2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3, lc[idx][j]);// 1x2 +1x1
	}
      }
      //internal->stack 
      lc[idx][j]=MIN2(lin[idx_3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di3+dj3+2*iext_s, lc[idx][j]); //
      //bulge -> stack
      lc[idx][j]=MIN2(lbx[idx_2][j+1]+di2+dj1+bext, lc[idx][j]);
      //min2=by[i][j+1];
      lc[idx][j]=MIN2(lby[idx_1][j+2]+di1+dj2+bext, lc[idx][j]);
      lc[idx][j]+=bonus_2;
      //if(j<=const5end){
      temp=min_colonne;
      min_colonne=MIN2(lc[idx][j]+(type>2?P->TerminalAU:0)+
		       P->dangle3[rtype[type]][SS1[i+1]]+
		       (!(structure[j-2]=='|') ? P->dangle5[rtype[type]][SS2[j-1]]: 0), 
		       min_colonne);
      if(temp>min_colonne){
	min_j_colonne=j;
      }
      //}
      //---------------------------------------------------------------------end update
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    }
    position[i+delta]=min_colonne;min_colonne=INF;
    position_j[i+delta]=min_j_colonne;
    i++;
  }
  free(S1); free(S2); free(SS1); free(SS2);  
  if(max<threshold+constthreshold){
    find_max_CXS(position, position_j, delta, threshold+constthreshold, constthreshold, alignment_length, s1, s2, access_s1, access_s2, fast, structure);  
  }
  if(max<constthreshold){
    plot_max_CXS(max, max_pos, max_pos_j,alignment_length, s1, s2, access_s1, access_s2,fast,structure);
  }
  for (i=1; i<=3; i++) {free(lc[i]);free(lin[i]);free(lbx[i]);free(lby[i]);}
  free(lc[0]);free(lin[0]);free(lbx[0]);free(lby[0]);
  free(lc);free(lin);free(lbx);free(lby);
  free(position);
  free(position_j);
  return NULL;
}

PRIVATE void find_max_CXS(const int *position, const int *position_j,const int delta, const int threshold, const int constthreshold, const int alignment_length, const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast, const char* structure){
  int pos=n1-9;
  if(fast==1){
    while(10 < pos--){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){                        		
	int search_range;                                           	
	search_range=delta+1;                                       	
	while(--search_range){   
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;                                  	
	  }                                                         	
	}                                                           	
	pos-=temp_min;                                              	
	int max_pos_j;                                              	
	max_pos_j=position_j[pos+delta];
	int max;
	max=position[pos+delta];
	printf("target upper bound %d: query lower bound %d  (%5.2f) \n", pos-10, max_pos_j-10, ((double)max)/100);
	pos=MAX2(10,pos-delta);
      }
    }
  }
  else{
    pos=n1-9;
    while( pos-- > 10 ){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min; //position on i
	int max_pos_j;
	max_pos_j=position_j[pos+delta]; //position on j
	//int begin_t=MAX2(9, pos-alignment_length);
	//int end_t  =MIN2(n1-10, pos);
	//int begin_q=MAX2(9, max_pos_j-2);
	//int end_q  =MIN2(n2-10, max_pos_j+alignment_length-2);
	int begin_t=MAX2(9,pos-alignment_length);
	int end_t  =pos;
	int begin_q=max_pos_j-2;
	int end_q  =MIN2(n2-9,max_pos_j+alignment_length-2);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
	char *local_structure = (char*) space(sizeof(char) * ( end_q - begin_q +2));
	strncpy(s3, (s1+begin_t),  end_t - begin_t+1);
	strncpy(s4, (s2+begin_q) , end_q - begin_q+1 );
	strncpy(local_structure, (structure+begin_q), end_q - begin_q +1);
	s3[end_t -begin_t +1 ]='\0';
	s4[end_q -begin_q +1 ]='\0';
	local_structure[end_q - begin_q +1]='\0';
	duplexT test;
	test = duplexfold_CXS(s3,s4,access_s1,access_s2,pos, max_pos_j,threshold,local_structure);
	if(test.energy * 100 < (threshold - constthreshold)){
	  int l1=strchr(test.structure, '&')-test.structure;
	  int dL = strrchr(structure,'|') - strchr(structure,'|');
	  dL+=1;
	  if(dL <=  strlen(test.structure)-l1-1){
	    printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n", test.structure, 
		   test.tb,test.te,test.qb,test.qe, test.ddG, test.energy, test.dG1, test.dG2);
	    pos=MAX2(10,pos-delta);
	  }
	}
	free(s3);free(s4);
	free(test.structure);
	free(local_structure);
      }
    }
  }
}

//PRIVATE int compare(const void *sub1, const void *sub2) {
//  int d;
//  if (((duplexT *) sub1)->ddG > ((duplexT *) sub2)->ddG)
//    return 1;
//  if (((duplexT *) sub1)->ddG < ((duplexT *) sub2)->ddG)
//    return -1;
//  d = ((duplexT *) sub1)->i - ((duplexT *) sub2)->i;
//  if (d!=0) return d;
//  return  ((duplexT *) sub1)->j - ((duplexT *) sub2)->j;
//}

PRIVATE void plot_max_CXS(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int ** access_s1, const int ** access_s2, const int fast, const char* structure)
{
  if(fast==1){ 
    printf("target upper bound %d: query lower bound %d (%5.2f)\n", max_pos-3, max_pos_j, ((double)max)/100); 
  } 
  else{ 
    int begin_t=MAX2(9,max_pos-alignment_length);
    int end_t  =max_pos;
    int begin_q=max_pos_j-2;
    int end_q  =MIN2(n2-9,max_pos_j+alignment_length-2);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
    char *local_structure = (char*) space(sizeof(char)*(end_q - begin_q +2));
    strncpy(s3, (s1+begin_t),  end_t - begin_t+1);
    strncpy(s4, (s2+begin_q) , end_q - begin_q+1 );
    strncpy(local_structure, (structure+begin_q) , end_q - begin_q +1 );
    s3[end_t -begin_t +1 ]='\0';
    s4[end_q -begin_q +1 ]='\0';
    local_structure[end_q - begin_q +1]='\0';
    duplexT test;
    test = duplexfold_CXS(s3,s4,access_s1,access_s2,max_pos, max_pos_j,INF,local_structure);
    int l1=  strchr(test.structure, '&')-test.structure;
    int dL = strrchr(structure,'|') - strchr(structure,'|');
    dL+=1;
    if(dL<=strlen(test.structure)-l1-1){
      printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n", test.structure, 
	     test.tb,test.te,test.qb,test.qe, test.ddG, test.energy, test.dG1, test.dG2);
    }
    free(s3);free(s4);free(test.structure);free(local_structure);
    
  }
}


/*---------------------------------------------------------duplexfold----------------------------------------------------------------------------------*/


PRIVATE duplexT duplexfold_C(const char *s1, const char *s2, const int extension_cost, const char* structure ) {
  int i, j, l1, Emin=INF, i_min=0, j_min=0;
  char *struc;
  duplexT mfe;
  int bonus=-10000;
  int *previous_const; //for each "|" constraint returns the position of the next "|" constraint
  
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6)) {
    update_fold_params();  P = scale_parameters();
    make_pair_matrix();
  }
  previous_const=(int *) space(sizeof(int) * (n4+1));
  j=n4+1;
  previous_const[j-1]=n4;
  int prev_temp = n4;
  while(--j){
    if(structure[j-1]=='|'){
      previous_const[j-1]=prev_temp;
      prev_temp=j;
    }
    else{
      previous_const[j-1]=prev_temp;
    }
  }
  c = (int **) space(sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++) c[i] = (int *) space(sizeof(int) * (n4+1));
  encode_seqs(s1, s2);  
  for (i=1; i<=n3; i++) {
    for (j=n4; j>0; j--) {
      int type, type2, E, k,l;
      int bonus_2 = (structure[j-1]=='|'? bonus: 0);
      type = pair[S1[i]][S2[j]];
      c[i][j] = type ? P->DuplexInit +2 * extension_cost + bonus_2: INF;
      if (!type) continue;
      if (i>1)  c[i][j] += P->dangle5[type][SS1[i-1]]+ extension_cost;
      if (j<n4 && !(structure[j]=='|')) c[i][j] += P->dangle3[type][SS2[j+1]]+ extension_cost;
      if (type>2) c[i][j] += P->TerminalAU;
      for (k=i-1; k>0 && k>i-MAXLOOP-2; k--) {
	for (l=j+1; l<=previous_const[j]; l++) {
	  if (i-k+l-j-2>MAXLOOP) break;
	  type2 = pair[S1[k]][S2[l]];
	  if (!type2) continue;
	  E = LoopEnergy(i-k-1, l-j-1, type2, rtype[type],
			 SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1])+(i-k+l-j)*extension_cost + bonus_2;
	  c[i][j] = MIN2(c[i][j], c[k][l]+E);
	}
      }
      E = c[i][j]; 
      if (i<n3) E += P->dangle3[rtype[type]][SS1[i+1]]+extension_cost;
      if (j>1 && !(structure[j-2]=='|'))  E += P->dangle5[rtype[type]][SS2[j-1]]+extension_cost;
      if (type>2) E += P->TerminalAU;
      if (E<Emin) {
	Emin=E; i_min=i; j_min=j;
      } 
    }
  }  
  struc = backtrack_C(i_min, j_min, extension_cost,structure,&Emin);
  if (i_min<n3) i_min++;
  if (j_min>1 ) j_min--;
  l1 = strchr(struc, '&')-struc;
  int size;
  size=strlen(struc)-1;
  Emin-= size * (extension_cost);  
  mfe.i = i_min;
  mfe.j = j_min;
  mfe.energy = (double) Emin/100.;
  mfe.structure = struc;
  free(previous_const); 
  if (!delay_free) {
    for (i=0; i<=n3; i++) free(c[i]);
    
    free(c);
    free(S1); free(S2); free(SS1); free(SS2);
  }
  return mfe;
}

PRIVATE char *backtrack_C(int i, int j, const int extension_cost, const char* structure, int *Emin) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0, *previous_const;
  char *st1, *st2, *struc;
  int bonus=-10000;
  previous_const=(int *) space(sizeof(int) * (n4+1)); //encodes the position of the constraints
  int j_temp=n4+1;
  previous_const[j_temp-1]=n4;
  int prev_temp = n4;
  while(--j_temp){
    if(structure[j_temp-1]=='|'){
      previous_const[j_temp-1]=prev_temp;
      prev_temp=j_temp;
    }
    else{
      previous_const[j_temp-1]=prev_temp;
    }
  }
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  i0=MIN2(i+1,n3); j0=MAX2(j-1,1);
  while (i>0 && j<=n4) {
    int bonus_2 = (structure[j-1]== '|'? bonus: 0);
    E = c[i][j]; traced=0;
    st1[i-1] = '(';
    st2[j-1] = ')'; 
    type = pair[S1[i]][S2[j]];
    if (!type) nrerror("backtrack failed in fold duplex a");
    for (k=i-1; k>0 && k>i-MAXLOOP-2; k--) {
      for (l=j+1; l<=previous_const[j]; l++) {
	int LE;
	if (i-k+l-j-2>MAXLOOP) break;
	type2 = pair[S1[k]][S2[l]];
	if (!type2) continue;
	LE = LoopEnergy(i-k-1, l-j-1, type2, rtype[type],
			SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1])+(i-k+l-j)*extension_cost + bonus_2;
	if (E == c[k][l]+LE) { 
	  *Emin-=bonus_2;
	  traced=1; 
	  i=k; j=l;
	  break;
	}
      }
      if (traced) break;
    }
    if (!traced) { 
      if (i>1) E -= P->dangle5[type][SS1[i-1]]+extension_cost;
      if (j<n4 && !(structure[j]=='|')) E -= P->dangle3[type][SS2[j+1]]+ extension_cost;
      //if (j<n4) E -= P->dangle3[type][SS2[j+1]]+extension_cost;
      if (type>2) E -= P->TerminalAU;
      if (E != P->DuplexInit+2*extension_cost + bonus_2) {
	nrerror("backtrack failed in fold duplex b");
      } else {
	*Emin-=bonus_2;
	break;
      }
    }
  }
  if (i>1)  i--;
  if (j<n4) j++;
  
  struc = (char *) space(i0-i+1+j-j0+1+2);
  for (k=MAX2(i,1); k<=i0; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j0; k<=j; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i-1,0)); strcat(struc, "&"); 
  strcat(struc, st2+j0-1);
  
  /* printf("%s %3d,%-3d : %3d,%-3d\n", struc, i,i0,j0,j);  */
  free(st1); free(st2);
  free(previous_const);
  return struc;
}




duplexT ** Lduplexfold_C(const char *s1, const char *s2, const int threshold, const int extension_cost, const int alignment_length, const int delta, const int fast, const char* structure, const int il_a, const int il_b, const int b_a, const int b_b)
{
  //duplexT test = duplexfold_C(s1, s2, extension_cost,structure);
  
  int i, j;
  int bopen=b_b;
  int bext=b_a+extension_cost;
  int iopen=il_b;
  int iext_s=2*(il_a+extension_cost);//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a+extension_cost;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j=10;
  int temp;
  int min_j_colonne=11;
  int max=INF;
  int bonus = -10000;
  int constthreshold=0; //minimal threshold corresponding to a structure complying to all constraints
  i=0;
  while(structure[i]!='\0'){
    if(structure[i]=='|') constthreshold+=bonus;
    i++;
  }    
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1
  //int nsubopt=10;  //total number of subopt
  int *position; //contains the position of the hits with energy > E
  int *position_j;
  //  int const5end; //position of the 5'most constraint. Only interaction reaching this position are taken into account.
  //const5end = strchr(structure,'|') - structure;
  //const5end++;
  n1 = (int) strlen(s1);
  n2 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  position = (int *) space ((delta+n1+3+delta) * sizeof(int));
  position_j= (int *) space ((delta+n1+3+delta) * sizeof(int));
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
  update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  lc = (int **) space(sizeof(int *) * 4);
  lc[0] = (int *) space(sizeof(int) * (n2+5)); 
  lc[1] = (int *) space(sizeof(int) * (n2+5));
  lc[2] = (int *) space(sizeof(int) * (n2+5));
  lc[3] = (int *) space(sizeof(int) * (n2+5));

/*   local int array initialisation-------------------------------------------*/
  lin = (int**) space (sizeof(int *) * 4);
  lin[0] = (int *) space(sizeof(int) * (n2+5));
  lin[1] = (int *) space(sizeof(int) * (n2+5));
  lin[2] = (int *) space(sizeof(int) * (n2+5));
  lin[3] = (int *) space(sizeof(int) * (n2+5));

/*   local bx array initialization--------------------------------------------*/ 
  lbx = (int**) space (sizeof(int *) * 4);
  lbx[0] = (int *) space(sizeof(int) * (n2+5));
  lbx[1] = (int *) space(sizeof(int) * (n2+5));
  lbx[2] = (int *) space(sizeof(int) * (n2+5));
  lbx[3] = (int *) space(sizeof(int) * (n2+5));

/*   local by array initialization--------------------------------------------*/ 
  lby = (int**) space (sizeof(int *) * 4);
  lby[0] = (int *) space(sizeof(int) * (n2+5));
  lby[1] = (int *) space(sizeof(int) * (n2+5));
  lby[2] = (int *) space(sizeof(int) * (n2+5));
  lby[3] = (int *) space(sizeof(int) * (n2+5));

  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(j=n2;j>=0;j--) {
   /*  --------------------------------------------------------local array*/
    lbx[0][j]=lbx[1][j]=lbx[2][j]=lbx[3][j]=INF;//no bulge before 1
    lin[0][j]=lin[1][j]=lin[2][j]=lin[3][j]=INF;//no in before  1
    lc[0][j] =lc[1][j] =lc[2][j] = lc[3][j]=INF; //no bulge and no in before n2
    lby[0][j]=lby[1][j]=lby[2][j]=lby[3][j]=INF;
    

  }
  /*--------------------------------------------------------local array*/

  
  //-------------------------------------------------------------matrix initialisierung
  i=10;
  i_length= n1 - 9 ;
  while(i < i_length) {
    int idx=i%4;
    int idx_1=(i-1)%4;
    int idx_2=(i-2)%4;
    int idx_3=(i-3)%4; 
    j=n2-9;
    while (9 < --j) {
      int bonus_2 = (structure[j-1]=='|' ? bonus : 0) ;
      int type, type2; 
      type = pair[S1[i]][S2[j]];
      lc[idx][j]=type ? P->DuplexInit + 2*extension_cost + bonus_2 : INF; //to avoid that previous value influence result should actually not be erforderlich
      if(!bonus_2){
	type2=pair[S2[j+1]][S1[i-1]];
	lin[idx][j]=MIN2(lc[idx_1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s, lin[idx_1][j]+iext_ass); //Iij=ci-1j+1+M+iopen+2iext
	lin[idx][j]=MIN2(lin[idx][j],lin[idx][j+1]+iext_ass);
	lin[idx][j]=MIN2(lin[idx][j],lin[idx_1][j+1]+iext_s);
	type2=pair[S2[j+1]][S1[i]];
	lby[idx][j]=MIN2(lby[idx][j+1]+bext, lc[idx][j+1]+bopen+bext+(type2>2?P->TerminalAU:0));
      }
      else{
	lin[idx][j] = lby[idx][j] = INF; //all loop containing "|" are rejected
      }
      type2=pair[S2[j]][S1[i-1]];
      lbx[idx][j]=MIN2(lbx[idx_1][j]+bext, lc[idx_1][j]+bopen+bext+(type2>2?P->TerminalAU:0));
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;}
      lc[idx][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+extension_cost+(structure[j]=='|'? 0 : P->dangle3[type][SS2[j+1]]+extension_cost);
      //type > 2 -> no GC or CG pair
      //------------------------------------------------------------------update c  matrix 
      // Be careful, no lc may come from a region where a "|" is in a loop, avoided in lin = lby = INF ... jedoch fuer klein loops muss man aufpassen ..
      type2=pair[S1[i-1]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_1][j+1]+P->stack[type2][rtype[type]]+2*extension_cost, lc[idx][j]);
      type2=pair[S1[i-2]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,lc[idx][j]);
      //kleine loops checks wird in den folgenden if test gemacht.
      if(!(structure[j]=='|')){
	type2=pair[S1[i-1]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,lc[idx][j]);
	type2=pair[S1[i-2]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+4*extension_cost, lc[idx][j]);
	type2 = pair[S1[i-3]][S2[j+2]];
	lc[idx][j]=MIN2(lc[idx_3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+5*extension_cost, lc[idx][j]);
	if(!(structure[j+1]=='|')){
	  type2 = pair[S1[i-3]][S2[j+3]];
	  lc[idx][j]=MIN2(lc[idx_3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+6*extension_cost,lc[idx][j]);
	  type2 = pair[S1[i-2]][S2[j+3]];
	  lc[idx][j]=MIN2(lc[idx_2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+5*extension_cost, lc[idx][j]);
	}
      }
      //internal->stack 
      lc[idx][j]=MIN2(lin[idx_3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*extension_cost+2*iext_s, lc[idx][j]); //
      //bulge -> stack
      lc[idx][j]=MIN2(lbx[idx_2][j+1]+2*extension_cost+bext, lc[idx][j]);
      //min2=by[i][j+1];
      lc[idx][j]=MIN2(lby[idx_1][j+2]+2*extension_cost+bext, lc[idx][j]);
      lc[idx][j]+=bonus_2;
      //      if(j<=const5end){
	temp=min_colonne;        
	min_colonne=MIN2(lc[idx][j]+(type>2?P->TerminalAU:0)+
			 P->dangle3[rtype[type]][SS1[i+1]]+extension_cost+
			 (!(structure[j-2]=='|') ? P->dangle5[rtype[type]][SS2[j-1]]+extension_cost : 0), 
			 min_colonne);
	if(temp>min_colonne){
	  min_j_colonne=j;
	  //	}
      }
      //---------------------------------------------------------------------end update      
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    } 
    position[i+delta]=min_colonne;min_colonne=INF;
    position_j[i+delta]=min_j_colonne;
    i++;
  }  
  free(S1); free(S2); free(SS1); free(SS2);  
  if(max<threshold+constthreshold){
    find_max_C(position, position_j, delta, threshold+constthreshold, constthreshold, alignment_length, s1, s2, extension_cost, fast, structure);
  }
  if(max<constthreshold){
    plot_max_C(max, max_pos, max_pos_j,alignment_length, s1, s2, extension_cost,fast,structure);
  }
  for (i=1; i<=3; i++) {free(lc[i]);free(lin[i]);free(lbx[i]);free(lby[i]);}
  free(lc[0]);free(lin[0]);free(lbx[0]);free(lby[0]);
  free(lc);free(lin);free(lbx);free(lby);
  free(position);
  free(position_j);
  return NULL;
}


PRIVATE void find_max_C(const int *position, const int *position_j,const int delta, const int threshold, const int constthreshold, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast,const char* structure){
  int pos=n1-9;
  if(fast==1){
    while(10 < pos--){
      int temp_min=0;                                               		
      if(position[pos+delta]<(threshold)){                          		
	int search_range;                                           		
	search_range=delta+1;                                       		
	while(--search_range){                                      		
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){		
	    temp_min=search_range;                                  		
	  }                                                         		
	}                                                           		
	pos-=temp_min;                                              		
	int max_pos_j;                                              		
	max_pos_j=position_j[pos+delta];
	int max;
	max=position[pos+delta];
	printf("target upper bound %d: query lower bound %d  (%5.2f) \n", pos-10, max_pos_j-10, ((double)max)/100);
	pos=MAX2(10,pos-delta);
      }
    }
  }
  else{
    pos=n1-9;
    while(10 < pos--){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min;
	int max_pos_j;
	max_pos_j=position_j[pos+delta];
	/* max_pos_j und pos entsprechen die realen position
	   in der erweiterten sequenz. 
	   pos=1 -> position 1 in the sequence (and not 0 like in C)
	   max_pos_j -> position 1 in the sequence ( not 0 like in C)
	*/
	int begin_t=MAX2(11, pos-alignment_length+1);
	int end_t  =MIN2(n1-10, pos+1);
	int begin_q=MAX2(11, max_pos_j-1);
	int end_q  =MIN2(n2-10, max_pos_j+alignment_length-2);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
	char *local_structure = (char*) space(sizeof(char)*(end_q - begin_q +2));
	strncpy(s3, (s1+begin_t-1),  end_t - begin_t +1);
	strncpy(s4, (s2+begin_q-1) , end_q - begin_q +1);
	strncpy(local_structure, (structure+begin_q-1) , end_q - begin_q +1 );
	s3[end_t -begin_t +1 ]='\0';
	s4[end_q -begin_q +1 ]='\0';
	local_structure[end_q - begin_q +1]='\0';
	duplexT test;
	test = duplexfold_C(s3, s4, extension_cost,local_structure);
	if(test.energy * 100 < (threshold-constthreshold)){
	  int l1=strchr(test.structure, '&')-test.structure;
	  int dL = strrchr(structure,'|') - strchr(structure,'|');
	  dL+=1;
	  if(dL <=  strlen(test.structure)-l1-1){
	    printf("%s %3d,%-3d : %3d,%-3d (%5.2f)\n", test.structure, 
		   begin_t-10+test.i-l1, 
		   begin_t-10+test.i-1, 
		   begin_q-10 + test.j-1 , 
		   (begin_q -11) + test.j + strlen(test.structure)-l1-2, 
		   test.energy);
	    pos=MAX2(10,pos-delta);
	  }
	}
	free(s3);free(s4);
	free(test.structure);
	free(local_structure);
      }
    }
  }
}
PRIVATE void plot_max_C(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast,const char* structure)
{
  if(fast==1){
    printf("target upper bound %d: query lower bound %d (%5.2f)\n", max_pos-10, max_pos_j-10, ((double)max)/100);
  }
  else{
    duplexT test;
    int begin_t=MAX2(11, max_pos-alignment_length+1);
    int end_t  =MIN2(n1-10, max_pos+1);
    int begin_q=MAX2(11, max_pos_j-1);
    int end_q  =MIN2(n2-10, max_pos_j+alignment_length-2);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
    char *local_structure = (char*) space(sizeof(char)*(end_q - begin_q +2));
    strncpy(s3, (s1+begin_t-1),  end_t - begin_t + 1);
    strncpy(s4, (s2+begin_q-1) , end_q - begin_q +1 );
    strncpy(local_structure, (structure+begin_q-1) , end_q - begin_q +1 );
    s3[end_t -begin_t +1 ]='\0';
    s4[end_q -begin_q +1 ]='\0';
    local_structure[end_q - begin_q +1]='\0';
    test = duplexfold_C(s3, s4, extension_cost,local_structure);
    int l1=strchr(test.structure, '&')-test.structure;
    int dL = strrchr(structure,'|') - strchr(structure,'|');
    dL+=1;
    if(dL <=  strlen(test.structure)-l1-1){
      printf("%s %3d,%-3d : %3d,%-3d (%5.2f)\n", test.structure, 
	     begin_t-10+test.i-l1, begin_t-10+test.i-1, begin_q-10 +test.j-1 , 
	     (begin_q -11) + test.j + strlen(test.structure)-l1-2  , test.energy);
      free(s3);free(s4);free(test.structure);
    }
    free(local_structure);
  }
}


/*---------------------------------UTILS------------------------------------------*/

//PRIVATE int print_struct(duplexT const *dup) {
//  /*this function print out the structure of a hybridization site
//    and return the value from which one can begin to look for the next
//    non-overlappig hybrid*/
//  
//  int l1;
//  int energy=(100*dup->energy);
//  int init=dup->end;
//  l1 = strchr(dup->structure, '&')-dup->structure;
//  printf("%s %3d,%-3d : %3d,%-3d %d\n", dup->structure, init-l1-1,
//	 init-2, dup->j+l1+2-strlen(dup->structure), dup->j, energy);
//  return init-l1-1;
//}

PRIVATE void update_dfold_params(void)
{
  P = scale_parameters();
  make_pair_matrix();
}

/*---------------------------------------------------------------------------*/

//PRIVATE int compare_XS(const void *sub1, const void *sub2) {
//  int d;
//  if (((duplexT *) sub1)->ddG > ((duplexT *) sub2)->ddG)
//    return 1;
//  if (((duplexT *) sub1)->ddG < ((duplexT *) sub2)->ddG)
//    return -1;
//  d = ((duplexT *) sub1)->te - ((duplexT *) sub2)->te;
//  if (d!=0) return d;
//  return  ((duplexT *) sub1)->tb - ((duplexT *) sub2)->tb;
//}

PRIVATE void encode_seqs(const char *s1, const char *s2) {
  unsigned int i,l;

  l = strlen(s1);
  S1 = encode_seq(s1);
  SS1= (short *) space(sizeof(short)*(l+1));
  /* SS1 exists only for the special X K and I bases and energy_set!=0 */
  
  for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
    SS1[i] = alias[S1[i]];   /* for mismatches of nostandard bases */
  }

  l = strlen(s2);
  S2 = encode_seq(s2);
  SS2= (short *) space(sizeof(short)*(l+1));
  /* SS2 exists only for the special X K and I bases and energy_set!=0 */
  
  for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
    SS2[i] = alias[S2[i]];   /* for mismatches of nostandard bases */
  }
}

//PRIVATE void my_encode_seq(const char *s1, const char *s2) {
//  unsigned int i,l;
//
//  l = strlen(s1);
//  S1 = (short *) space(sizeof(short)*(l+4));//XXX l+1 -> l+2
//  SS1= (short *) space(sizeof(short)*(l+4));//XXX l+1 -> l+2
//  /* SS1 exists only for the special X K and I bases and energy_set!=0 */
//  //XXX i=1->i=2 l+1->l+2
//  for (i=3; i<=l+3; i++) { /* make numerical encoding of sequence */
//    S1[i]= (short) encode_char(toupper(s1[i-3]));//XXX i-1 -> i-2
//    SS1[i] = alias[S1[i]];   /* for mismatches of nostandard bases */
//  }
//
//  l = strlen(s2);
//  S2 = (short *) space(sizeof(short)*(l+4));
//  SS2= (short *) space(sizeof(short)*(l+4));
//  /* SS2 exists only for the special X K and I bases and energy_set!=0 */
//  
//  for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
//    S2[i]= (short) encode_char(toupper(s2[i-1]));
//    SS2[i] = alias[S2[i]];   /* for mismatches of nostandard bases */
//  }
//}

PRIVATE short * encode_seq(const char *sequence) {
  unsigned int i,l;
  short *S;
  l = strlen(sequence);
  S = (short *) space(sizeof(short)*(l+2));
  S[0] = (short) l;

  /* make numerical encoding of sequence */
  for (i=1; i<=l; i++)
    S[i]= (short) encode_char(toupper(sequence[i-1]));

  /* for circular folding add first base at position n+1 */
  S[l+1] = S[1];

  return S;
}

//int arraySize(duplexT** array)
//{
//  int site_count=0;
//  while(array[site_count]!=NULL){
//    site_count++;
//  }
//  return site_count;
//}
//

//void freeDuplexT(duplexT** array)
//{
//  int size=arraySize(array);
//  while(--size){
//    free(array[size]->structure);
//    free(array[size]);
//  }
//  free(array[0]->structure);
//  free(array);
//}
//


///------OLD WORKING DUPLEXFOLD_XS PART


/* duplexT duplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int i_pos, const int j_pos,const int dangle_i, const int dangle_j, const int threshold) { */
/*   int i, j,p,q, l1, Emin=INF, l_min=0, k_min=0; */
/*   char *struc; */
/*   struc=NULL; */
/*   duplexT mfe; */
/*   n3 = (int) strlen(s1); */
/*   n4 = (int) strlen(s2); */
/*   if ((!P) || (fabs(P->temperature - temperature)>1e-6)) { */
/*     update_fold_params();  P = scale_parameters(); */
/*     make_pair_matrix(); */
/*   } */
  
/*   c = (int **) space(sizeof(int *) * (n3+1)); */
/*   for (i=0; i<=n3; i++) c[i] = (int *) space(sizeof(int) * (n4+1)); */
/*   for (i=0; i<=n3; i++){ */
/* 	for(j=0;j<=n4;j++){ */
/* 		c[i][j]=INF; */
/*         } */
/*   } */
/*   encode_seqs(s1, s2); */
/*   int type, type2, type3, E, k,l; */
/*   i=n3-dangle_i; j=1+dangle_j; */
/*   if(dangle_i && dangle_j){ */
/* 	  type = pair[S1[i]][S2[j]]; */
/* 	  c[i][j] = P->DuplexInit;  */
/* 	  c[i][j]+= P->dangle3[rtype[type]][SS1[i+1]]+P->dangle5[rtype[type]][SS2[j-1]]; */
/*   } else if(dangle_i){ */
/* 	type = pair[S1[i]][S2[j]];        */
/*         c[i][j] = P->DuplexInit;  */
/*         c[i][j]+= P->dangle3[rtype[type]][SS1[i+1]]; */
/*   } else if(dangle_j){ */
/* 	type = pair[S1[i]][S2[j]];                */
/*         c[i][j] = P->DuplexInit;  */
/*         c[i][j]+= P->dangle5[rtype[type]][SS2[j-1]]; */
/*   }else{ */
/* 	type = pair[S1[i]][S2[j]];  */
/* 	c[i][j] = P->DuplexInit;  */
/*   } */
/*   if (type>2) c[i][j] += P->TerminalAU; */
/*   for (k=n3-dangle_i-1; k>0 ; k--) { */
/*         c[k+1][0]=INF; */
/* 	for (l=1+dangle_j; l<=n4; l++) { */
/*                 c[k][l]=INF; */
/* 	  	type2 = pair[S1[k]][S2[l]]; */
/* 	  	if (!type2) continue; */
/* 		for (p=k+1; p<n3-dangle_i+1 && p<k+MAXLOOP-1; p++) {	  	 */
/* 			for (q = l-1; q > dangle_j; q--) { */
/* 				if (p-k+l-q-2>MAXLOOP) break; */
/* 				type3=pair[S1[p]][S2[q]]; */
/* 				if(!type3) continue; */
/* 				E = LoopEnergy(p-k-1, l-q-1, type2, rtype[type3],SS1[k+1], SS2[l-1], SS1[p-1], SS2[q+1]); */
/* 	  			c[k][l] = MIN2(c[k][l], c[p][q]+E); */
/* 			} */
/* 		} */
/*       		E = c[k][l];  */
/* 	        if (type2>2) E += P->TerminalAU; */
/* 		E+=access_s1[n3-dangle_i-k+1][i_pos+2+dangle_i]+access_s2[l-(1+dangle_j)+1][j_pos + l-(1+dangle_j)]; */
/* 		if (k>1) E += P->dangle5[type2][SS1[k-1]]; */
/* 	 	if (l<n4) E += P->dangle3[type2][SS2[l+1]];		 */
/* 	        if (E<Emin) { */
/* 	        	Emin=E; k_min=k; l_min=l; */
/*       		}  */
/*     	} */
/*   } */
/*   mfe.energy = c[k_min][l_min]; */
/*   type2=pair[S1[k_min]][S2[l_min]]; */
/*   if (k_min>1) mfe.energy  += P->dangle5[type2][SS1[k_min-1]]; */
/*   if (l_min<n4) mfe.energy += P->dangle3[type2][SS2[l_min+1]];	 */
/*   if (type2>2) mfe.energy  += P->TerminalAU; */
/*   if(Emin > threshold || mfe.energy >0){  */
/* 	mfe.structure=NULL;  */
/* 	for (i=0; i<=n3; i++) free(c[i]); */
/*     	free(c); */
/* 	free(S1); free(S2); free(SS1); free(SS2); */
/* 	return mfe; */
/*   } else{  */
/* 	  struc = backtrack_XS(k_min, l_min, access_s1, access_s2); */
/*   } */
/*   mfe.dG1=access_s1[n3-dangle_i-k_min+1][i_pos+2+dangle_i]; */
/*   mfe.dG2=access_s2[l_min-(1+dangle_j)+1][j_pos + l_min-(1+dangle_j)]; */
 /*   mfe.ddG =Emin; */
/*   if (k_min>1)  k_min--;  */
/*   if (l_min<n4 ) l_min++; */
/*   mfe.structure = struc; */
/*     for (i=0; i<=n3; i++) free(c[i]); */
/*     free(c); */
/*     free(S1); free(S2); free(SS1); free(SS2); */
/*   return mfe; */
/* } */


/* PRIVATE char *backtrack_XS(int i, int j, const int **access_s1,const int **access_s2 ) { */
/*   /\* backtrack structure going backwards from i, and forwards from j  */
/*      return structure in bracket notation with & as separator *\/ */
/*   int k, l, type, type2, E, traced, i0, j0; */
/*   char *st1, *st2, *struc; */
/*   st1 = (char *) space(sizeof(char)*(n3+1)); */
/*   st2 = (char *) space(sizeof(char)*(n4+1)); */
/*   i0=MAX2(i-1,1);j0=MIN2(j+1,n4); */
/*   while (i<=n3 && j>=1) { */
/*     E = c[i][j]; traced=0; */
/*     st1[i-1] = '('; */
/*     st2[j-1] = ')';  */
/*     type = pair[S1[i]][S2[j]]; */
/*     if (!type) nrerror("backtrack failed in fold duplex bli"); */
/*     for (k=i+1; k<=n3 && k>i-MAXLOOP-2; k++) { */
/*       for (l=j-1; l>=1; l--) { */
/* 	int LE; */
/* 	if (i-k+l-j-2>MAXLOOP) break; */
/* 	type2 = pair[S1[k]][S2[l]]; */
/* 	if (!type2) continue; */
/* 	LE = LoopEnergy(k-i-1, j-l-1, type, rtype[type2], SS1[i+1], SS2[j-1], SS1[k-1], SS2[l+1]); */
/* 	if (E == c[k][l]+LE) { */
/* 	  traced=1;  */
/* 	  i=k; j=l; */
/* 	  break; */
/* 	} */
/*       } */
/*       if (traced) break; */
/*     } */
/*     if (!traced) {  */
/*       if (i<n3) E -= P->dangle3[rtype[type]][SS1[i+1]];//+access_s1[1][i+1]; */
/*       if (j>1)  E -= P->dangle5[rtype[type]][SS2[j-1]];//+access_s2[1][j+1]; */
/*       if (type>2) E -= P->TerminalAU; */
/*       break; */
/*       if (E != P->DuplexInit) { */
/*         nrerror("backtrack failed in fold duplex bal"); */
/*       } else break; */
/*     } */
/*   } */
/*   if (i<n3)  i++; */
/*   if (j>1)   j--; */
/*   struc = (char *) space(i-i0+1+j0-j+1+2); */
/*   for (k=MAX2(i0,1); k<=i; k++) if (!st1[k-1]) st1[k-1] = '.'; */
/*   for (k=j; k<=j0; k++) if (!st2[k-1]) st2[k-1] = '.'; */
/*   strcpy(struc, st1+MAX2(i0-1,0)); strcat(struc, "&");  */
/*   strcat(struc, st2+j-1); */
/*   free(st1); free(st2); */
/*   return struc; */
/* } */


/* duplexT** Lduplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int threshold, const int alignment_length, const int delta, const int fast)//, const int target_dead, const int query_dead) */
/* { */
  
/*   int i, j; */
/*   int bopen=200; */
/*   int bext=40; */
/*   int iopen=170; */
/*   int iext_s=2*8;//iext_s 2 nt nucleotide extension of interior loop, on i and j side */
/*   int iext_ass=50+8;//iext_ass assymetric extension of interior loop, either on i or on j side. */
/*   int min_colonne=INF; //enthaelt das maximum einer kolonne */
/*   int i_length; */
/*   int max_pos;//get position of the best hit */
/*   int max_pos_j; */
/*   int temp; */
/*   int min_j_colonne; */
/*   int max=INF; */
/*   //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1 */
/*   int nsubopt=10;  //total number of subopt */
/*   int *position; //contains the position of the hits with energy > E */
/*   int *position_j; */
/*   n1 = (int) strlen(s1); */
/*   n2 = (int) strlen(s2); */
/*   //delta_check is the minimal distance allowed for two hits to be accepted */
/*   //if both hits are closer, reject the smaller ( in term of position)  hits  */
/*   position = (int *) space ((delta+n1+3+delta) * sizeof(int)); */
/*   position_j= (int *) space ((delta+n1+3+delta) * sizeof(int)); */
/*   //i want to implement a function that, given a position in a long sequence and a small sequence, */
/*   //duplexfold them at this position and report the result at the command line// */
/*   //for this i first need to rewrite backtrack in order to remove the printf functio */
/*   //END OF DEFINITION FOR NEEDED SUBOPT DATA  */
  
/*   if ((!P) || (fabs(P->temperature - temperature)>1e-6)) */
/*     update_dfold_params(); */
/*   /\*local c array initialization---------------------------------------------*\/ */
/*   lc = (int **) space(sizeof(int *) * 4); */
/*   lc[0] = (int *) space(sizeof(int) * (n2+5));  */
/*   lc[1] = (int *) space(sizeof(int) * (n2+5)); */
/*   lc[2] = (int *) space(sizeof(int) * (n2+5)); */
/*   lc[3] = (int *) space(sizeof(int) * (n2+5)); */
  
/*   /\*   local int array initialisation-------------------------------------------*\/ */
/*   lin = (int**) space (sizeof(int *) * 4); */
/*   lin[0] = (int *) space(sizeof(int) * (n2+5)); */
/*   lin[1] = (int *) space(sizeof(int) * (n2+5)); */
/*   lin[2] = (int *) space(sizeof(int) * (n2+5)); */
/*   lin[3] = (int *) space(sizeof(int) * (n2+5)); */
  
/*   /\*   /\*local bx array initialization--------------------------------------------*\/  */
/*   lbx = (int**) space (sizeof(int *) * 4); */
/*   lbx[0] = (int *) space(sizeof(int) * (n2+5)); */
/*   lbx[1] = (int *) space(sizeof(int) * (n2+5)); */
/*   lbx[2] = (int *) space(sizeof(int) * (n2+5)); */
/*   lbx[3] = (int *) space(sizeof(int) * (n2+5)); */
  
/*   /\*   /\*local by array initialization--------------------------------------------*\/  */
/*   lby = (int**) space (sizeof(int *) * 4); */
/*   lby[0] = (int *) space(sizeof(int) * (n2+5)); */
/*   lby[1] = (int *) space(sizeof(int) * (n2+5)); */
/*   lby[2] = (int *) space(sizeof(int) * (n2+5)); */
/*   lby[3] = (int *) space(sizeof(int) * (n2+5)); */
  
/*   int di1; */
/*   int di2; */
/*   int di3; */
/*   int di4;  */
  
  
  
/*   /\*-------------------------------------------------------------------------*\/ */
/*   /\*end of array initialisation----------------------------------*\/ */
/*   /\*maybe int *** would be better*\/ */
/*   encode_seqs(s1,s2); */
/*   //------------------------------------------matrix initialisierung */
/*   for(j=n2+4;j>=0;j--) { */
/*     /\*  /\*--------------------------------------------------------local array*\/ */
/*     lbx[0][j]=lbx[1][j]=lbx[2][j]=lbx[3][j]=INF;//no bulge before 1 */
/*     lin[0][j]=lin[1][j]=lin[2][j]=lin[3][j]=INF;//no in before  1 */
/*     lc[0][j] =lc[1][j] =lc[2][j] = lc[3][j]=INF; //no bulge and no in before n2 */
/*     lby[0][j]=lby[1][j]=lby[2][j]=lby[3][j]=INF; */
/*   } */
/*   /\*--------------------------------------------------------local array*\/ */
  
  
/*   //-------------------------------------------------------------matrix initialisierung */
/*   i=10 /\*target_dead*\/; //start from 2 (	i=4) because no structure allowed to begin with a single base pair */
/*   i_length= n1 - 9  /\*- target_dead*\/ ;  */
/*   while(i < i_length) { */
/*     int idx=i%4; */
/*     int idx_1=(i-1)%4; */
/*     int idx_2=(i-2)%4; */
/*     int idx_3=(i-3)%4; */
    
/*     j=n2 - 9 /\*- query_dead*\/; //start from n2-1 because no structure allow to begin with a single base pair  */
/*     /\*     switch(i){ *\/ */
/*     /\*     case 3: //i=1 *\/ */
/*     /\*       di1 = access_s1[1][i];  *\/ */
/*     /\*       di2 = 0;	      *\/ */
/*     /\*       di3 = 0;	      *\/ */
/*     /\*       di4 = 0; *\/ */
/*     /\*       break;  *\/ */
/*     /\*     case 4://i=2 *\/ */
/*     /\*       di1 = access_s1[2][i]   - access_s1[1][i-1];		      *\/ */
/*     /\*       di2 = access_s1[1][i-1] + di1;	      *\/ */
/*     /\*       di3 = 0;	      *\/ */
/*     /\*       di4 = 0; *\/ */
/*     /\*       break;  *\/ */
/*     /\*     case 5://i=3 *\/ */
/*     /\*       di1 = access_s1[3][i]   - access_s1[2][i-1];		      *\/ */
/*     /\*       di2 = access_s1[2][i-1] - access_s1[1][i-2] + di1;	      *\/ */
/*     /\*       di3 = access_s1[1][i-2] + di2;	      *\/ */
/*     /\*       di4 = 0;   *\/ */
/*     /\*       break;                                      *\/ */
/*     /\*     case 6://i=3					  *\/ */
/*     /\*       di1 = access_s1[4][i]   - access_s1[3][i-1];	  *\/ */
/*     /\*       di2 = access_s1[3][i-1] - access_s1[2][i-2] + di1;  *\/ */
/*     /\*       di3 = access_s1[2][i-2] - access_s1[1][i-3] + di2;	     			  *\/ */
/*     /\*       di4 = access_s1[1][i-3] + di3; *\/ */
/*     /\*       break;                                        *\/ */
/*     /\*     case 7://i=3					     *\/ */
/*     /\*       di1 = access_s1[5][i]   - access_s1[4][i-1];	     *\/ */
/*     /\*       di2 = access_s1[4][i-1] - access_s1[3][i-2] + di1;     *\/ */
/*     /\*       di3 = access_s1[3][i-2] - access_s1[3][i-3] + di2;     *\/ */
/*     /\*       di4 = access_s1[2][i-3] - access_s1[1][i-4] + di3; *\/ */
/*     /\*       break;                     *\/ */
/*     /\*     case 8://i=3					     *\/ */
/*     /\*       di1 = access_s1[5][i]   - access_s1[4][i-1];	     *\/ */
/*     /\*       di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;     *\/ */
/*     /\*       di3 = access_s1[4][i-2] - access_s1[3][i-3] + di2;     *\/ */
/*     /\*       di4 = access_s1[3][i-3] - access_s1[2][i-4] + di3;         *\/ */
/*     /\*       break;         *\/ */
/*     /\*     case 9://i=3					     *\/ */
/*     /\*       di1 = access_s1[5][i]   - access_s1[4][i-1];	     *\/ */
/*     /\*       di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;     *\/ */
/*     /\*       di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2;     *\/ */
/*     /\*       di4 = access_s1[4][i-3] - access_s1[3][i-4] + di3;    *\/ */
/*     /\*       break;       *\/ */
/*     /\*     default: *\/ */
/*     /\*       di1 = access_s1[5][i]   - access_s1[4][i-1];		      *\/ */
/*     /\*       di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;	      *\/ */
/*     /\*       di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2;	      *\/ */
/*     /\*       di4 = access_s1[5][i-3] - access_s1[4][i-4] + di3; *\/ */
/*     /\*     } *\/ */
/*     di1 = access_s1[5][i]   - access_s1[4][i-1];	    */
/*     di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1; */
/*     di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2; */
/*     di4 = access_s1[5][i-3] - access_s1[4][i-4] + di3; */




/*       di1=MIN2(di1,165); */
/*       di2=MIN2(di2,330); */
/*       di3=MIN2(di3,495); */
/*     while (--j > 9/\*query_dead - 1*\/) { */
/*              //----------------------------------------------------------update lin lbx lby matrix */
/*       int dj1,dj2,dj3,dj4; */
/* /\*       if(j>n2-7){ *\/ */
/* /\* 	      if(j==n2){ *\/ */
/* /\* 		dj1 = access_s2[1][j];	 *\/ */
/* /\* 		dj2 = 0; *\/ */
/* /\* 		dj3 = 0; *\/ */
/* /\* 		dj4 = 0; *\/ */
/* /\* 	      } else if(j==n2-1){ *\/ */
/* /\* 		dj1 = access_s2[2][j+1] - access_s2[1][j+1];	 *\/ */
/* /\* 		dj2 = access_s2[1][j+1] + dj1; *\/ */
/* /\* 		dj3 = 0; *\/ */
/* /\* 		dj4 = 0; *\/ */
/* /\* 	      } else if(j==n2-2){ *\/ */
/* /\* 		dj1 = access_s2[3][j+2] - access_s2[2][j+2];	 *\/ */
/* /\* 		dj2 = access_s2[2][j+2] - access_s2[1][j+2] + dj1; *\/ */
/* /\* 		dj3 = access_s2[1][j+2] + dj2; *\/ */
/* /\* 		dj4 = 0; *\/ */
/* /\* 	      } else if(j==n2-3){ *\/ */
/* /\* 		dj1 = access_s2[4][j+3] - access_s2[3][j+3];	 *\/ */
/* /\* 		dj2 = access_s2[3][j+3] - access_s2[2][j+3] + dj1; *\/ */
/* /\* 		dj3 = access_s2[2][j+3] - access_s2[1][j+2] + dj2; *\/ */
/* /\* 		dj4 = access_s2[1][j+3] + dj3; *\/ */
/* /\* 	      } else if(j==n2-4){ *\/ */
/* /\* 		dj1 = access_s2[5][j+4] - access_s2[4][j+4];	 *\/ */
/* /\* 		dj2 = access_s2[4][j+4] - access_s2[3][j+4] + dj1; *\/ */
/* /\* 		dj3 = access_s2[3][j+4] - access_s2[2][j+4] + dj2; *\/ */
/* /\* 		dj4 = access_s2[2][j+4] - access_s2[1][j+4] + dj3; *\/ */
/* /\* 	      } else if(j==n2-5){					     *\/ */
/* /\* 		dj1 = access_s2[5][j+4] - access_s2[4][j+4];	 *\/ */
/* /\* 		dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1; *\/ */
/* /\* 		dj3 = access_s2[4][j+5] - access_s2[3][j+5] + dj2; *\/ */
/* /\* 		dj4 = access_s2[3][j+5] - access_s2[2][j+5] + dj3; *\/ */
/* /\* 	      } else if(j==n2-6){ *\/ */
/* /\* 		dj1 = access_s2[5][j+4] - access_s2[4][j+4];	 *\/ */
/* /\* 		dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1; *\/ */
/* /\* 		dj3 = access_s2[5][j+6] - access_s2[4][j+6] + dj2; *\/ */
/* /\* 		dj4 = access_s2[4][j+6] - access_s2[3][j+6] + dj3; *\/ */
/* /\* 	      } *\/ */
/* /\* 	}  *\/ */
/* /\* 	else{ *\/ */
/* /\* 		dj1 = access_s2[5][j+4] - access_s2[4][j+4];	 *\/ */
/* /\* 		dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1; *\/ */
/* /\* 		dj3 = access_s2[5][j+6] - access_s2[4][j+6] + dj2; *\/ */
/* /\* 		dj4 = access_s2[5][j+7] - access_s2[4][j+7] + dj3; *\/ */
/* /\*       } *\/ */
/*       dj1 = access_s2[5][j+4] - access_s2[4][j+4];	 */
/*       dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1; */
/*       dj3 = access_s2[5][j+6] - access_s2[4][j+6] + dj2; */
/*       dj4 = access_s2[5][j+7] - access_s2[4][j+7] + dj3; */
/*       dj1=MIN2(dj1,165); */
/*       dj2=MIN2(dj2,330); */
/*       dj3=MIN2(dj3,495); */
/*       int type2; */
/*       type2=pair[S2[j+1]][S1[i-1]]; */
/*       lin[idx][j]=MIN2(lc[idx_1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+di1+dj1+iopen+iext_s,lin[idx_1][j]+iext_ass + di1);  */
/*       lin[idx][j]=MIN2(lin[idx][j],lin[idx][j+1]+iext_ass + dj1); //Iij=ci-1j+1+M+iopen+2iext */
/*       lin[idx][j]=MIN2(lin[idx][j],lin[idx_1][j+1]+iext_s + di1 + dj1); */
/*       type2=pair[S2[j]][S1[i-1]]; */
/*       lbx[idx][j]=MIN2(lbx[idx_1][j]+bext + di1, lc[idx_1][j]+bopen+bext+(type2>2?P->TerminalAU:0) + di1); */
/*       type2=pair[S2[j+1]][S1[i]]; */
/*       lby[idx][j]=MIN2(lby[idx][j+1]+bext + dj1 , lc[idx][j+1]+bopen+bext+(type2>2?P->TerminalAU:0)+ dj1); */
/*       //--------------------------------------------------------------- end update recursion */
/*       int type,temp; */
/*       type  = pair[S1[i]][S2[j]]; */
/*       if(!type){continue;}  */
/*       type2=pair[S1[i+1]][S2[j-1]]; */
/*       if(type2){ */
/*       	//----------------LOCAL																		    */
/* 	//initialize lc to 0;																		    */
/* 	lc[idx][j]= type2 ? P->DuplexInit + access_s1[1][i] + access_s2[1][j] : INF; //to avoid that previous value influence result should actually not be erforderlich */
/* 	lc[idx][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+P->dangle3[type][SS2[j+1]]; */
/*       } */
/* 	//} */
/*       //type > 2 -> no GC or CG pair */
/*       //------------------------------------------------------------------update c  matrix  */
/*       type2=pair[S1[i-1]][S2[j+1]];  */
/*       lc[idx][j]=MIN2(lc[idx_1][j+1]+P->stack[type2][rtype[type]]+di1+dj1, lc[idx][j]); //0x0+1x1 */
/*       type2=pair[S1[i-1]][S2[j+2]]; */
/*       lc[idx][j]=MIN2(lc[idx_1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2,lc[idx][j]);//1x0 + 1x1 */
/*       type2=pair[S1[i-2]][S2[j+1]]; */
/*       lc[idx][j]=MIN2(lc[idx_2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1,lc[idx][j]);//0x1 +1x1 */
/*       type2=pair[S1[i-2]][S2[j+2]]; */
/*       lc[idx][j]=MIN2(lc[idx_2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2, lc[idx][j]); // 1x1 +1x1 */
/*       type2 = pair[S1[i-3]][S2[j+3]]; */
/*       lc[idx][j]=MIN2(lc[idx_3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3,lc[idx][j]);//2x2 + 1x1 */
/*       type2 = pair[S1[i-3]][S2[j+2]]; */
/*       lc[idx][j]=MIN2(lc[idx_3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2, lc[idx][j]); // 2x1 +1x1 */
/*       type2 = pair[S1[i-2]][S2[j+3]]; */
/*       lc[idx][j]=MIN2(lc[idx_2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3, lc[idx][j]);// 1x2 +1x1 */
/*       //internal->stack  */
/*       lc[idx][j]=MIN2(lin[idx_2][j+2]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2+iext_s+iext_s, lc[idx][j]); // */
/*       //bulge -> stack */
/*       lc[idx][j]=MIN2(lbx[idx_2][j+1]+di2+dj1+bext, lc[idx][j]); */
/*       //min2=by[i][j+1]; */
/*       lc[idx][j]=MIN2(lby[idx_1][j+2]+di1+dj2+bext, lc[idx][j]); */
/*       temp=min_colonne; */
/*       min_colonne=MIN2(lc[idx][j]+(type>2?P->TerminalAU:0)+ */
/*                	       P->dangle3[rtype[type]][SS1[i+1]]+ */
/*                	       P->dangle5[rtype[type]][SS2[j-1]],  */
/*                	       min_colonne); */
/*       if(temp>min_colonne){ */
/* 	min_j_colonne=j; */
/*       } */
/*       //---------------------------------------------------------------------end update */
/*     } */
/*     if(max>=min_colonne){ */
/*       max=min_colonne; */
/*       max_pos=i; */
/*       max_pos_j=min_j_colonne; */
/*     } */
/*     position[i+delta]=min_colonne;min_colonne=INF; */
/*     position_j[i+delta]=min_j_colonne; */
/*     i++; */
/*   } */
/*   free(S1); free(S2); free(SS1); free(SS2);   */
/*   if(max<threshold){ */
/*     duplexT* subopt; */
/*     subopt=find_max_XS(position, position_j, delta, threshold, alignment_length, s1, s2, access_s1, access_s2, fast); */
/*     if(subopt!=NULL){ */
/*       int gothrough=0; */
/*       while((subopt+gothrough)->structure!=NULL){ */
/*       	duplexT* temp; */
/* 	temp=(subopt+gothrough); */
/* 	printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n", temp->structure, */
/* 	       temp->tb,temp->te,temp->qb, temp->qe, (double)temp->ddG*0.01, (double)temp->energy*0.01,  */
/* 	       (double)temp->dG1*0.01,(double)temp->dG2*0.01); */
/* 	       free(temp->structure); */
/* 	       gothrough++; */
/* 	} */
/*       free(subopt); */
/*     } */
/*   } */
/*   //plot_max_XS(max, max_pos, max_pos_j,alignment_length, s1, s2, access_s1, access_s2,fast); */
/*   for (i=1; i<=3; i++) {free(lc[i]);free(lin[i]);free(lbx[i]);free(lby[i]);} */
/*   free(lc[0]);free(lin[0]);free(lbx[0]);free(lby[0]); */
/*   free(lc);free(lin);free(lbx);free(lby); */
/*   free(position); */
/*   free(position_j); */
/*   return NULL; */
/* } */

/* PRIVATE duplexT* find_max_XS(const int *position, const int *position_j,const int delta, const int threshold, const int alignment_length, const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast){ */
  
/*   int n_max;n_max=16; */
/*   duplexT* subopt; */
/*   subopt=(duplexT *) space(n_max*sizeof(duplexT)); */
/*   int mposition, n_subopt; */
/*   n_subopt=0; */
/*   int init_r=0; */
/*   int init_l=0; */
/*   int dl; */
/*   int pos=n1-9; */
/*   if(fast==1){ */
/*     while(10 < pos--){ */
/*       printf("%d %d \n", pos, position[pos+delta]); */
/*       int temp_min=0; */
/*       if(position[pos+delta]<(threshold)){                        		 */
/* 	int search_range;                                           	 */
/* 	search_range=delta+1;                                       	 */
/* 	while(--search_range){    */
/* 	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){ */
/* 	    temp_min=search_range;                                  	 */
/* 	  }                                                         	 */
/* 	}                                                           	 */
/* 	pos-=temp_min;                                              	 */
/* 	int max_pos_j;                                              	 */
/* 	max_pos_j=position_j[pos+delta]; */
/* 	int max; */
/* 	max=position[pos+delta]; */
/* 	printf("target upper bound %d: query lower bound %d  (%5.2f) \n", pos-10, max_pos_j-10, ((double)max)/100); */
/* 	pos=MAX2(10,pos-alignment_length); */
/*       } */
/*     } */
/*     return NULL; */
/*   } */
/*   else{ */
/*     pos=n1-9; */
/*     while( pos-- > 10 ){ */
/*       duplexT test; */
/*       int temp_min=0; */
/*       if(position[pos+delta]<(threshold)){ */
/* 	int search_range; */
/* 	search_range=delta+1; */
/* 	while(--search_range){ */
/* 	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){ */
/* 	    temp_min=search_range; */
/* 	  } */
/* 	} */
/* 	pos-=temp_min; //position on i */
/* 	temp_min=0; */
/* 	int max_pos_j; */
/* 	max_pos_j=position_j[pos+delta]; //position on j */
/* 	int temp_pos, temp_max_pos_j; */
/* 	temp_pos=pos; */
/* 	temp_max_pos_j=max_pos_j; */
/* 	int dangle_i, dangle_j,begin,end; */
/* 	dangle_i=0; dangle_j=0; */
/* 	if(pos<n1-9) { */
/* 	  dangle_i=1; */
/* 	  temp_pos++; */
/* 	} */
/* 	begin=MAX2(3, pos-alignment_length-1+1); */
/* 	if(max_pos_j >1){ */
/* 	  dangle_j=1; */
/* 	  temp_max_pos_j--; */
/* 	} */
/* 	end=MIN2(n2, max_pos_j+alignment_length+1-1); */
/* 	char *s3 = (char*) space(sizeof(char)*(temp_pos-begin+2)); */
/* 	char *s4 = (char*) space(sizeof(char)*(end-temp_max_pos_j+2)); */
/* 	strncpy(s3, (s1+begin-3), temp_pos-begin+1); */
/* 	strncpy(s4, (s2+temp_max_pos_j-1), end-temp_max_pos_j+1); */
/* 	s3[temp_pos-begin+1]='\0'; */
/* 	s4[end-temp_max_pos_j+1]='\0'; */
/* 	int bla,l1; */
/* 	test = duplexfold_XS(s3, s4, access_s1, access_s2,pos-3, max_pos_j,dangle_i, dangle_j, threshold); */
/* 	if(test.structure==NULL){ */
/* 	  free(s3); free(s4); */
/* 	  continue; */
/* 	} */
/* 	l1=strchr(test.structure,'&')-test.structure; */
/* 	int dangle_begin, dangle_end; */
/* 	dangle_begin=0;dangle_end=0; */
/* 	if(test.structure[0]=='.'){dangle_begin=1;} */
/* 	if(test.structure[strlen(test.structure)-1]=='.'){dangle_end=1;} */
/* 	if (n_subopt+1>=n_max) { */
/* 	  n_max*=2; */
/* 	  subopt = (duplexT *) xrealloc(subopt, n_max*sizeof(duplexT)); */
/* 	} */
/* 	subopt[n_subopt].tb=temp_pos-l1-1;                                 */
/* 	subopt[n_subopt].te=temp_pos-2; */
/* 	subopt[n_subopt].qb=temp_max_pos_j; */
/* 	subopt[n_subopt].qe=temp_max_pos_j+strlen(test.structure)-l1-2; */
/* 	subopt[n_subopt].energy=test.energy;	 */
/* 	subopt[n_subopt].ddG=test.ddG; */
/* 	subopt[n_subopt].dG1=test.dG1; */
/* 	subopt[n_subopt].dG2=test.dG2; */
/* 	subopt[n_subopt++].structure=test.structure; */
/* 	//printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f +%5.2f +%5.2f)\n",  */
/* 	//       test.structure, */
/* 	//       temp_pos -l1-1, */
/* 	//       temp_pos-2,  */
/* 	//       temp_max_pos_j, */
/* 	//       temp_max_pos_j+strlen(test.structure)-l1-2, */
/* 	//       ((double)(test.energy + access_s1[l1-dangle_i-dangle_begin][pos] +access_s2[strlen(test.structure)-l1-1-dangle_j-dangle_end][max_pos_j + (strlen(test.structure) - l1-2 ) - dangle_end -dangle_j ] ))*0.01, */
/* 	//       test.energy/100, (double) access_s1[l1-dangle_i-dangle_begin][pos]*0.01 , (double)access_s2[strlen(test.structure)-l1-1-dangle_j-dangle_end][max_pos_j + (strlen(test.structure) - l1-2 ) - dangle_end -dangle_j ]*0.01 ); */
/* 	pos=MAX2(3,pos-1); */
/* 	free(s3);free(s4); */
/*       } */
/*     } */
/*   } */
/*   if (subopt_sorted) qsort(subopt, n_subopt, sizeof(duplexT), compare); */
  
/*   if (n_subopt+1>=n_max) { */
/*     n_max*=2; */
/*     subopt = (duplexT *) xrealloc(subopt, n_max*sizeof(duplexT)); */
/*   } */
    
/*     subopt[n_subopt].structure=NULL; */
/*     return subopt; */
/* } */

/* PRIVATE int compare(const void *sub1, const void *sub2) { */
/*   int d; */
/*   if (((duplexT *) sub1)->ddG > ((duplexT *) sub2)->ddG) */
/*     return 1; */
/*   if (((duplexT *) sub1)->ddG < ((duplexT *) sub2)->ddG) */
/*     return -1; */
/*   d = ((duplexT *) sub1)->i - ((duplexT *) sub2)->i; */
/*   if (d!=0) return d; */
/*   return  ((duplexT *) sub1)->j - ((duplexT *) sub2)->j; */
/* } */

/* PRIVATE void plot_max_XS(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int ** access_s1, const int ** access_s2, const int fast) */
/* { */

	
/* 	if(fast==1){ */
/* 		printf("target upper bound %d: query lower bound %d (%5.2f)\n", max_pos-3, max_pos_j, ((double)max)/100); */
/* 	} */
/* 	else{ */
/* 		duplexT test; */
/* 		int dangle_j; */
/* 		int dangle_i,l1; */
/* 		int temp_pos, temp_max_pos_j; */
/* 		temp_pos=max_pos; temp_max_pos_j=max_pos_j;  */
/* 		dangle_j=dangle_i=0; */
/* 		if(max_pos_j >1){ */
/* 	  		dangle_j=1; */
/* 	  		temp_max_pos_j--; */
/* 		} */
/* 		if(max_pos<n1+2) { */
/* 		  dangle_i=1; */
/* 		  temp_pos++; */
/* 		} */
/* 		int end=MIN2(n2, max_pos_j+alignment_length+1-1); */
/* 		int begin=MAX2(3, max_pos-alignment_length-1+1); */
/* 		char *s3 = (char*) space(sizeof(char)*(temp_pos-begin+2)); */
/* 		char *s4 = (char*) space(sizeof(char)*(end-temp_max_pos_j+2)); */
/* 		strncpy(s3, (s1+begin-3), temp_pos-begin+1); */
/* 		strncpy(s4, (s2+temp_max_pos_j-1), end-temp_max_pos_j+1); */
/* 		s3[temp_pos-begin+1]='\0'; */
/* 		s4[end-temp_max_pos_j+1]='\0'; */
/* 		test = duplexfold_XS(s3, s4, access_s1, access_s2,max_pos-3, max_pos_j,dangle_i, dangle_j, INF); */
/* 		if(test.structure==NULL){ */
/* 	  		return; */
/* 		} */
/* 		l1=strchr(test.structure,'&')-test.structure; */
/* 		int dangle_begin, dangle_end; */
/* 		dangle_begin=0;dangle_end=0; */
/* 		if(test.structure[0]=='.'){dangle_begin=1;} */
/* 		if(test.structure[strlen(test.structure)-1]=='.'){dangle_end=1;} */
/* 	       printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n",  */
/* 	       test.structure, */
/* 	       temp_pos-l1 - 1, */
/* 	       temp_pos-2,  */
/* 	       temp_max_pos_j, */
/* 	       temp_max_pos_j+strlen(test.structure)-l1-2, */
/* 	       ((double)(test.energy + access_s1[l1-dangle_i-dangle_begin][max_pos] +access_s2[strlen(test.structure)-l1-1-dangle_j-dangle_end][max_pos_j + (strlen(test.structure) - l1-2 ) - dangle_end -dangle_j ] ))*0.01, */
/* 	       test.energy/100, (double) access_s1[l1-dangle_i-dangle_begin][max_pos]*0.01 , (double)access_s2[strlen(test.structure)-l1-1-dangle_j-dangle_end][max_pos_j + (strlen(test.structure) - l1-2 ) - dangle_end -dangle_j ]*0.01 ); */

/* 		free(s3);free(s4);free(test.structure); */
/* 	} */
/* } */

