/* Last changed Time-stamp: <2007-10-30 14:06:22 htafer> */
/*                
	   compute the duplex structure of two RNA strands,
		allowing only inter-strand base pairs.
	 see cofold() for computing hybrid structures without
			     restriction.
			     Ivo Hofacker
			  Vienna RNA package

*/


/*
  library containing the function used in rnaplex
  the program rnaplex uses the following function
  Lduplexfold: finds high scoring segments
  it stores the end-position of these segments in an array 
  and call then for each of these positions the duplexfold function 
  which allows to make backtracking for each of the high scoring position
  It allows to find suboptimal partially overlapping (depends on a a parameter) 
  duplexes between a long RNA and a shorter one.
  Contrarly to RNAduplex, the energy model is not in E~log(N), 
  where N is the length of an interial loop but used an affine model, 
  where the extension and begin parameter are fitted to the energy
  parameter used by RNAduplex. This allows to check for duplex between a short RNA(20nt) 
  and a long one at the speed of 1Mnt/s. At this speed the whole genome (3Gnt) can be analyzed for one siRNA 
  in about 50 minutes.
  The algorithm is based on an idea by Durbin and Eddy:when the alginment reach a value larger than a
  given threshold this value is stored in an array. When the alignment score goes 
  then under this threshold, the alignemnent begin from this value, in that way the backtracking allow us 
  to find all non-overlapping high-scoring segments. 
  For more information check "durbin, biological sequence analysis"
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "utils.h"
#include "energy_par.h"
#include "fold_vars.h"
#include "fold.h"
#include "pair_mat.h"
#include "params.h"
#include "plex.h"
#include "ali_plex.h"
//#################SIMD###############//

/*@unused@*/
static char rcsid[] UNUSED = "$Id: plex.c,v 1.14 2007/06/12 12:50:16 htafer Exp $";
//int subopt_sorted=0;

#define PUBLIC
#define PRIVATE static

#define STACK_BULGE1  1   /* stacking energies for bulges of size 1 */
#define NEW_NINIO     1   /* new asymetry penalty */
#define ARRAY 32          /*array size*/
#define UNIT 100
#define MINPSCORE -2 * UNIT
//define indices for the Single Array approach defined in FLduplexfold_XS
//input is idx for i, j for j and the length of the query RNA
#define  LCI(i,j,l)    ((i     )*l + j)
#define LINI(i,j,l)    ((i +  4)*l + j)
#define LBXI(i,j,l)    ((i +  8)*l + j)
#define LBYI(i,j,l)    ((i + 12)*l + j)

PRIVATE void  encode_seqs(const char *s1, const char *s2);
PRIVATE short *encode_seq(const char *seq);
//PRIVATE void  my_encode_seq(const char *s1, const char *s2);
PRIVATE void  update_dfold_params(void);
//PRIVATE int   compare(const void *sub1, const void *sub2);
//PRIVATE int   compare_XS(const void *sub1, const void *sub2);
//PRIVATE duplexT* backtrack(int threshold, const int extension_cost);
//static void  print_struct(duplexT const *dup);

//PRIVATE int   print_struct(duplexT const *dup);
//PRIVATE int   get_rescaled_energy(duplexT const *dup);
PRIVATE duplexT duplexfold(const char *s1, const char *s2, const int extension_cost);
PRIVATE char * backtrack(int i, int j, const int extension_cost);
PRIVATE void   find_max(const int *position, const int *position_j, const int delta, const int threshold, const int length, const char *s1, const char *s2, const int extension_cost, const int fast,const int il_a, const int il_b, const int b_a, const int b_b);
PRIVATE void   plot_max(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast,const int il_a, const int il_b, const int b_a, const int b_b);

PRIVATE duplexT duplexfold_XS(const char *s1, const char *s2,const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int threshold);
PRIVATE char *   backtrack_XS(int i, int j, const int** access_s1, const int** access_s2);
PRIVATE void find_max_XS(const int *position, const int *position_j,const int delta, const int threshold, const int alignment_length, 
			 const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast,const int il_a, const int il_b, const int b_a, const int b_b);
PRIVATE void plot_max_XS(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast,const int il_a, const int il_b, const int b_a, const int b_b);

PRIVATE duplexT fduplexfold(const char *s1, const char *s2, const int extension_cost, const int il_a, const int il_b, const int b_a, const int b_b);
PRIVATE duplexT fduplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int threshold,const int il_a, const int il_b, const int b_a, const int b_b);
PRIVATE char * fbacktrack_XS(int i, int j, const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int il_a, const int il_b, const int b_a, const int b_b, int *dGe, int *dGeplex, int *dGx, int *dGy);
PRIVATE char *fbacktrack(int i, int j, const int extension_cost,const int il_a, const int il_b, const int b_a, const int b_b, int *dG);


/*@unused@*/

#define MAXSECTORS      500     /* dimension for a backtrack array */
#define LOCALITY        0.      /* locality parameter for base-pairs */

#define MIN2(A, B)      ((A) < (B) ? (A) : (B))
#define MAX2(A, B)      ((A) > (B) ? (A) : (B))

PRIVATE paramT *P = NULL;
PRIVATE int   **c, **in, **bx, **by;      /* energy array used in duplexfold */
PRIVATE int  **lc, **lin, **lbx, **lby;   /* energy array used in Lduplexfold
					     this arrays contains only 3 columns
					     In this way I reduce my memory use and
					     I can make most of my computation and 
					     accession in the computer cash
					     which is the main performance boost*/
					     


/*PRIVATE int last_cell;                    this variable is the last_cell containing
					    the information about the alignment
					    useful only if there is an alignment 
					    which extends till the last nucleotide of 
					    the long sequence*/

PRIVATE short  *S1, *SS1, *S2, *SS2;/*contains the sequences*/
PRIVATE int   n1,n2;    /* sequence lengths */
PRIVATE int n3, n4; /*sequence length for the duplex*/;
extern  int  LoopEnergy(int n1, int n2, int type, int type_2,
                        int si1, int sj1, int sp1, int sq1);
PRIVATE int delay_free=0;


/*-----------------------------------------------------------------------duplexfold_XS---------------------------------------------------------------------------*/

PRIVATE duplexT duplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int threshold) {
  int i, j,p,q, Emin=INF, l_min=0, k_min=0;
  char *struc;
  struc=NULL;
  duplexT mfe;
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);
  if ((!P) || (fabs(P->temperature - temperature)>1e-6)) {
    update_fold_params();  P = scale_parameters();
    make_pair_matrix();
  }
  
  c = (int **) space(sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++) c[i] = (int *) space(sizeof(int) * (n4+1));
  for (i=0; i<=n3; i++){
    for(j=0;j<=n4;j++){
      c[i][j]=INF;
    }
  }
  encode_seqs(s1, s2);
  int type, type2, type3, E, k,l;
  i=n3-1; j=2;
  type = pair[S1[i]][S2[j]]; 
  if(!type){
    printf("Error during initialization of the duplex in duplexfold_XS\n");
    mfe.structure=NULL;
    mfe.energy = INF;
    return mfe;
  }
  c[i][j] = P->DuplexInit; 
  if (type>2) c[i][j] += P->TerminalAU;
  c[i][j]+=P->dangle3[rtype[type]][SS1[i+1]];
  c[i][j]+=P->dangle5[rtype[type]][SS2[j-1]];
  for (k=i-1; k>0 ; k--) {
    c[k+1][0]=INF;
    for (l=j+1; l<=n4; l++) {
      c[k][l]=INF;
      type2 = pair[S1[k]][S2[l]];
      if (!type2) continue;
      for (p=k+1; p< n3 && p<k+MAXLOOP-1; p++) {	  	
	for (q = l-1; q > 1; q--) {
	  if (p-k+l-q-2>MAXLOOP) break;
	  type3=pair[S1[p]][S2[q]];
	  if(!type3) continue;
	  E = LoopEnergy(p-k-1, l-q-1, type2, rtype[type3],SS1[k+1], SS2[l-1], SS1[p-1], SS2[q+1]);
	  c[k][l] = MIN2(c[k][l], c[p][q]+E);
	}
      }
      E = c[k][l]; 
      if (type2>2) E += P->TerminalAU;
      E+=access_s1[i-k+1][i_pos]+access_s2[l-1][j_pos+(l-1)-1];
      if (k>1) E += P->dangle5[type2][SS1[k-1]]; 
      if (l<n4) E += P->dangle3[type2][SS2[l+1]];
      if (E<Emin) {
	Emin=E; k_min=k; l_min=l;
      } 
    }
  }
  //  mfe.energy =((float)  c[k_min][l_min])/100; //mfe.energy without dangle                                                                                                        
  //  mfe.energy=Emin;
  //type2=pair[S1[k_min]][S2[l_min]];
  //if (type2>2) Emin  += P->TerminalAU;

  if(Emin  > threshold){
    mfe.energy=INF;
    mfe.ddG=INF;
    mfe.structure=NULL;
    for (i=0; i<=n3; i++) free(c[i]);
    free(c);
    free(S1); free(S2); free(SS1); free(SS2);
    return mfe;
  } else{
    struc = backtrack_XS(k_min, l_min, access_s1, access_s2);
  }


//  type2=pair[S1[k_min]][S2[l_min]];
//  if (type2>2) Emin  += P->TerminalAU;
//  
//  if(Emin > threshold || mfe.energy >0){ 
//    mfe.structure=NULL; 
//    for (i=0; i<=n3; i++) free(c[i]);
//    free(c);
//    free(S1); free(S2); free(SS1); free(SS2);
//    return mfe;
//  } else{ 
//    struc = backtrack_XS(k_min, l_min, access_s1, access_s2);
//  }
  //lets take care of the dangles
  //find best combination 
  int dx_5, dx_3, dy_5, dy_3,dGx,dGy,bonus_x;
  dx_5=0; dx_3=0; dy_5=0; dy_3=0;dGx=0;dGy=0;bonus_x=0;
  //x--------x
  // ||||||||
  //x--------x
   dGx = access_s1[i-k_min+1][i_pos];dx_3=0; dx_5=0;bonus_x=0; 
/*   if(dGx + bonus_x > P->dangle3[rtype[type]][SS1[i+1]] + access_s1[i-k_min+2][i_pos+1]){ */
/*     dGx =  access_s1[i-k_min+2][i_pos+1]; */
/*     bonus_x = P->dangle3[rtype[type]][SS1[i+1]]; */
/*     dx_3=1; */
/*     dx_5=0; */
/*   } */
/*   if(dGx  > P->dangle5[type2][SS1[k_min-1]] + access_s1[i-k_min+2][i_pos]) { */
/*     dx_5=1; */
/*     dx_3=0; */
/*     bonus_x = P->dangle5[type2][SS1[k_min-1]]; */
/*     dGx = access_s1[i-k_min+2][i_pos]; */
/*   } */
/*   if(dGx > P->dangle5[type2][SS1[k_min-1]] + P->dangle3[rtype[type]][SS1[i+1]] + access_s1[i-k_min+3][i_pos+1]){ */
/*     dx_5=dx_3=1; */
/*     bonus_x = P->dangle5[type2][SS1[k_min-1]] + P->dangle3[rtype[type]][SS1[i+1]] ; */
/*     dGx = access_s1[(n3-2)-k_min+3][i_pos+1]; */
/*   } */
   dGy = access_s2[l_min-j+1][j_pos + (l_min-1)-1]; 
/*   bonus_y = dy_5 = dy_3 = 0; */
/*   if(dGy + bonus_y > P->dangle5[rtype[type]][SS2[(l_min-1)-1]] +  */
/*      access_s2[l_min-1+1][j_pos  + (l_min -1) -1  ]){ */
/*     dGy = access_s2[l_min-1+1][j_pos -1 + (l_min -1) -1]; */
/*     bonus_y = P-> dangle5[rtype[type]][SS2[(l_min-1)-1]]; */
/*     dy_3 = 0; */
/*     dy_5 = 1; */
/*   } */
/*   if(dGy + bonus_y > P->dangle3[type2][SS2[l_min+1]] + access_s2[l_min-1+1][j_pos + 1 + (l_min-1) -1]){ */
/*     dGy = access_s2[l_min-1+1][j_pos +1 +(l_min-1) -1]; */
/*     bonus_y = P->dangle3[type2][SS2[l_min+1]]; */
/*     dy_3=0; */
/*     dy_5=1; */
/*   } */
/*   if(dGy+bonus_y > P->dangle3[type2][SS2[l_min+1]] + P->dangle5[rtype[type]][SS2[(l_min-1)-1]] + */
/*      access_s2[l_min-1+2][j_pos  + 1 + (l_min -1) -1]){ */
/*     dGy = access_s2[l_min-1+2][j_pos  + 1 + l_min -1 -1]; */
/*     bonus_y = P->dangle3[type2][SS2[l_min+1]] + P-> dangle5[rtype[type]][SS2[(l_min-1)-1]]; */
/*     dy_3=1; */
/*     dy_5=1; */
/*   } */
  mfe.tb=i_pos -9 - i + k_min -1 -dx_5;
  mfe.te=i_pos -9 -1 + dx_3;
  mfe.qb=j_pos -9 -1 - dy_5;
  mfe.qe=j_pos + l_min -3 -9 + dy_3;
  mfe.ddG=(double) Emin * 0.01;
  mfe.dG1=(double) dGx*0.01 ;
  mfe.dG2=(double) dGy*0.01 ; 
  //mfe.energy += bonus_y + bonus_x;
  mfe.energy= mfe.ddG - mfe.dG1 - mfe.dG2;

  mfe.structure = struc;
  for (i=0; i<=n3; i++) free(c[i]);
  free(c);
  free(S1); free(S2); free(SS1); free(SS2);
  return mfe;
}


PRIVATE char *backtrack_XS(int i, int j, const int **access_s1,const int **access_s2 ) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0;
  char *st1, *st2, *struc;
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  i0=i;/*MAX2(i-1,1);*/j0=j;/*MIN2(j+1,n4);*/
  while (i<=n3-1 && j>=2) {
    E = c[i][j]; traced=0;
    st1[i-1] = '(';
    st2[j-1] = ')'; 
    type = pair[S1[i]][S2[j]];
    if (!type) nrerror("backtrack failed in fold duplex bli");
    for (k=i+1; k<=n3 && k>i-MAXLOOP-2; k++) {
      for (l=j-1; l>=1; l--) {
	int LE;
	if (i-k+l-j-2>MAXLOOP) break;
	type2 = pair[S1[k]][S2[l]];
	if (!type2) continue;
	LE = LoopEnergy(k-i-1, j-l-1, type, rtype[type2], SS1[i+1], SS2[j-1], SS1[k-1], SS2[l+1]);
	if (E == c[k][l]+LE) {
	  traced=1; 
	  i=k; j=l;
	  break;
	}
      }
      if (traced) break;
    }
    if (!traced) { 
      if (i<n3) E -= P->dangle3[rtype[type]][SS1[i+1]];//+access_s1[1][i+1];
      if (j>1)  E -= P->dangle5[rtype[type]][SS2[j-1]];//+access_s2[1][j+1];
      if (type>2) E -= P->TerminalAU;
      break;
      if (E != P->DuplexInit) {
        nrerror("backtrack failed in fold duplex bal");
      } else break;
    }
  }
  //if (i<n3)  i++;
  //if (j>1)   j--;
  struc = (char *) space(i-i0+1+j0-j+1+2);
  for (k=MAX2(i0,1); k<=i; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j; k<=j0; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i0-1,0)); strcat(struc, "&"); 
  strcat(struc, st2+j-1);
  free(st1); free(st2);
  return struc;
}

PRIVATE duplexT fduplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int i_pos, const int j_pos, const int threshold,const int il_a, const int il_b, const int b_a, const int b_b) {
  int i, j, Emin, i_min, j_min,l1;
  duplexT mfe;
  char *struc;
  int bopen=b_b;
  int bext=b_a;
  int iopen=il_b;
  int iext_s=2*il_a;//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  int temp;
  int min_j_colonne;
  int max=INF;
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1

  int *position; //contains the position of the hits with energy > E
  int *position_j;
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
  update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  c  = (int**)  space(sizeof(int *) * (n3+1));
  in = (int**) space (sizeof(int *) * (n3+1));
  bx = (int**) space (sizeof(int *) * (n3+1));
  by = (int**) space (sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++){
    c[i]  = (int *) space(sizeof(int) * (n4+1));  
    in[i] = (int *) space(sizeof(int) * (n4+1));  
    bx[i] = (int *) space(sizeof(int) * (n4+1));  
    by[i] = (int *) space(sizeof(int) * (n4+1));  
  }
  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(i=8; i<11;i++){
    for(j=n4-7;j>=8;j--) {
      in[i][j]=INF;//no in before  1
      c[i][j] =INF; //no bulge and no in before n2
      bx[i][j]=INF;//no bulge before 1
      by[i][j]=INF;
 
      in[n3-i+1][j]=INF;//no in before  1
      c[n3-i+1][j] =INF; //no bulge and no in before n2
      bx[n3-i+1][j]=INF;//no bulge before 1
      by[n3-i+1][j]=INF;
    }
  }
  for(j=8; j < 11; j++) {
    for(i=10; i<n3-9; i++){
      in[i][j]=INF;//no in before  1
      c[i][j] =INF; //no bulge and no in before n2
      bx[i][j]=INF;//no bulge before 1
      by[i][j]=INF;
      
      in[i][n4-j+1]=INF;//no in before  1
      c[i][n4-j+1] =INF; //no bulge and no in before n2
      bx[i][n4-j+1]=INF;//no bulge before 1
      by[i][n4-j+1]=INF;
    }
  }
  /*--------------------------------------------------------local array*/
  
  
  //-------------------------------------------------------------matrix initialisierung
  i=11;
  i_length=n3-9;
  while(i < i_length) {
    int di1,di2,di3,di4;
    int idiff=i_pos-(n3-10-i);
    di1 = access_s1[5][idiff]   - access_s1[4][idiff-1];	   
    di2 = access_s1[5][idiff-1] - access_s1[4][idiff-2] + di1;
    di3 = access_s1[5][idiff-2] - access_s1[4][idiff-3] + di2;
    di4 = access_s1[5][idiff-3] - access_s1[4][idiff-4] + di3;
    di1=MIN2(di1,165);
    di2=MIN2(di2,330);
    di3=MIN2(di3,495); 
    j=n4-9;
    min_colonne=INF;
    while (10 < --j) {
      int dj1,dj2,dj3,dj4;
      int jdiff=j_pos+j-11;
      dj1 = access_s2[5][jdiff+4] - access_s2[4][jdiff+4];	
      dj2 = access_s2[5][jdiff+5] - access_s2[4][jdiff+5] + dj1;
      dj3 = access_s2[5][jdiff+6] - access_s2[4][jdiff+6] + dj2;
      dj4 = access_s2[5][jdiff+7] - access_s2[4][jdiff+7] + dj3;
      dj1=MIN2(dj1,165);
      dj2=MIN2(dj2,330);
      dj3=MIN2(dj3,495);
      int type, type2; 
      type = pair[S1[i]][S2[j]];
      //----------------LOCAL
      //initialize lc to 0;
      c[i][j]=type ? P->DuplexInit + access_s1[1][idiff]+access_s2[1][jdiff] : INF; //to avoid that previous value influence result should actually not be erforderlich
      //----------------------------------------------------------update lin bx by matrix
      type2=pair[S2[j+1]][S1[i-1]];
      in[i][j]=MIN2(c[i - 1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s+di1+dj1, in[i - 1][j]+iext_ass + di1); //Iij=ci-1j+1+M+iopen+2iext
      in[i][j]=MIN2(in[i][j],in[i][j+1]+iext_ass + dj1);
      in[i][j]=MIN2(in[i][j],in[i - 1][j+1]+iext_s + di1 + dj1);
      type2=pair[S2[j]][S1[i-1]];
      bx[i][j]=MIN2(bx[i - 1][j]+bext + di1, c[i - 1][j]+bopen+bext+(type2>2?P->TerminalAU:0) + di1);
      type2=pair[S2[j+1]][S1[i]];
      by[i][j]=MIN2(by[i][j+1]+bext+dj1, c[i][j+1]+bopen+bext+(type2>2?P->TerminalAU:0)+dj1);
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;}
      c[i][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+P->dangle3[type][SS2[j+1]];
      //type > 2 -> no GC or CG pair
      //------------------------------------------------------------------update c  matrix 
      type2=pair[S1[i-1]][S2[j+1]];
      c[i][j]=MIN2(c[i - 1][j+1]+P->stack[type2][rtype[type]]+di1+dj1, c[i][j]);
      type2=pair[S1[i-1]][S2[j+2]];
      c[i][j]=MIN2(c[i - 1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2,c[i][j]);
      type2=pair[S1[i-2]][S2[j+1]];
      c[i][j]=MIN2(c[i - 2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1,c[i][j]);
      type2=pair[S1[i-2]][S2[j+2]];
      c[i][j]=MIN2(c[i - 2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2, c[i][j]);
      type2 = pair[S1[i-3]][S2[j+3]];
      c[i][j]=MIN2(c[i - 3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3,c[i][j]);
      type2 = pair[S1[i-3]][S2[j+2]];
      c[i][j]=MIN2(c[i - 3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2, c[i][j]);
      type2 = pair[S1[i-2]][S2[j+3]];
      c[i][j]=MIN2(c[i - 2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3, c[i][j]);
      //internal->stack 
      //!!      c[i][j]=MIN2(in[i - 2][j+2]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+iext_s+di2+dj2, c[i][j]); //
      c[i][j]=MIN2(in[i - 3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*iext_s+di3+dj3, c[i][j]);
      //bulge -> stack
      c[i][j]=MIN2(bx[i - 2][j+1]+di2+dj1+bext, c[i][j]);
      //min2=by[i][j+1];
      c[i][j]=MIN2(by[i - 1][j+2]+di1+dj2+bext, c[i][j]);
      temp=min_colonne;
      min_colonne=MIN2(c[i][j]+(type>2?P->TerminalAU:0)+
		       P->dangle3[rtype[type]][SS1[i+1]]+
		       P->dangle5[rtype[type]][SS2[j-1]], 
		       min_colonne);
      if(temp>min_colonne){
	min_j_colonne=j;
      }
      //---------------------------------------------------------------------end update
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    }
    i++;
  }
  Emin=max;
  if(Emin>threshold){
    free(S1); free(S2); free(SS1); free(SS2);  
    for (i=0; i<=n3; i++) {
      free(c[i]);
      free(in[i]);
      free(bx[i]);
      free(by[i]);
    }
    free(c);free(in);free(bx);free(by);
    mfe.energy=0;
    mfe.structure=NULL;
    return mfe;
  }
  i_min=max_pos;
  j_min=max_pos_j;
  int dGe, dGeplex, dGx, dGy;
  dGe=dGeplex=dGx=dGy=0;
  struc = fbacktrack_XS(i_min, j_min, access_s1, access_s2, i_pos, j_pos,il_a, il_b,b_a,b_b,&dGe, &dGeplex, &dGx, &dGy);

  l1 = strchr(struc, '&')-struc;
  int size;
  size=strlen(struc)-1;
  int lengthx; int endx; int lengthy; int endy;
  lengthx=l1;
  lengthx-=(struc[0]=='.'?1:0);
  lengthx-=(struc[l1-1]=='.'?1:0);
  endx=(i_pos-(n3-i_min));
  lengthy=size-l1;
  lengthy-=(struc[size]=='.'?1:0);
  lengthy-=(struc[l1+1]=='.'?1:0);
  endy=j_pos+j_min+lengthy -22;
  if (i_min<n3-10) i_min++;
  if (j_min>11 ) j_min--;
  mfe.i = i_min;
  mfe.j = j_min;
  mfe.ddG = (double) Emin*0.01;
  mfe.structure = struc;
  mfe.energy_backtrack = (double) dGe * 0.01;
  mfe.energy = (double) dGeplex * 0.01;
  mfe.opening_backtrack_x = (double) dGx * 0.01; 
  mfe.opening_backtrack_y = (double) dGy * 0.01;
  mfe.dG1=(double) access_s1[lengthx][endx+10] * 0.01;
  mfe.dG2=(double) access_s2[lengthy][endy+10] * 0.01;
  free(S1); free(S2); free(SS1); free(SS2);  
  for (i=0; i<=n3; i++) {
    free(c[i]);
    free(in[i]);
    free(bx[i]);
    free(by[i]);
  }
  free(c);free(in);free(bx);free(by);
  return mfe;
}

PRIVATE char *fbacktrack_XS(int i, int j, const int** access_s1, const int** access_s2, const int i_pos, const int j_pos,const int il_a, const int il_b, const int b_a, const int b_b, int *dG, int *dGplex, int *dGx, int *dGy) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0;
  char *st1, *st2, *struc;
  int bopen=b_b;
  int bext=b_a;
  int iopen=il_b;
  int iext_s=2*il_a;//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a;//iext_ass assymetric extension of interior loop, either on i or on j side.
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  i0=MIN2(i+1,n3-10); j0=MAX2(j-1,11);
  int state;
  state=1; //we start backtracking from a a pair , i.e. c-matrix
  /* state 1 -> base pair, c
     state 2 -> interior loop, in
     state 3 -> bx loop, bx
     state 4 -> by loop, by
  */
  traced=1;
  k=i; l=j; //stores the i,j information for subsequence usage see *
  int idiff,jdiff;
  type = pair[S1[i]][S2[j]];
  *dG+=(type>2?P->TerminalAU:0)+P->dangle3[rtype[type]][SS1[i+1]]+P->dangle5[rtype[type]][SS2[j-1]];
  *dGplex=*dG;
  while (i>10 && j<=n4-9 && traced) {
    int di1,di2,di3,di4;
    idiff=i_pos-(n3-10-i);
    di1 = access_s1[5][idiff]   - access_s1[4][idiff-1];	   
    di2 = access_s1[5][idiff-1] - access_s1[4][idiff-2] + di1;
    di3 = access_s1[5][idiff-2] - access_s1[4][idiff-3] + di2;
    di4 = access_s1[5][idiff-3] - access_s1[4][idiff-4] + di3;
    di1=MIN2(di1,165);
    di2=MIN2(di2,330);
    di3=MIN2(di3,495); 
    int dj1,dj2,dj3,dj4;
    jdiff=j_pos+j-11;
    dj1 = access_s2[5][jdiff+4] - access_s2[4][jdiff+4];	
    dj2 = access_s2[5][jdiff+5] - access_s2[4][jdiff+5] + dj1;
    dj3 = access_s2[5][jdiff+6] - access_s2[4][jdiff+6] + dj2;
    dj4 = access_s2[5][jdiff+7] - access_s2[4][jdiff+7] + dj3;
    dj1=MIN2(dj1,165);
    dj2=MIN2(dj2,330);
    dj3=MIN2(dj3,495);
    traced=0;
    switch(state){
    case 1:
      type = pair[S1[i]][S2[j]];
      if(!type) nrerror("backtrack failed in fold duplex");
      type2=pair[S1[i-1]][S2[j+1]];
      if(type2 && c[i][j]== (c[i - 1][j+1]+P->stack[type2][rtype[type]]+di1+dj1)){
	k=i-1;
	l=j+1;
	(*dG)+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di1;
	*dGy+=dj1;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-1]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2)){
	k=i-1;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di1;
	*dGy+=dj2;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-2]][S2[j+1]];
      if(type2 && c[i][j]==(c[i - 2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1)){
	k=i-2;
	l=j+1;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di2;
	*dGy+=dj1;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-2]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2)){
	k=i-2;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di2;
	*dGy+=dj2;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-3]][S2[j+3]];
      if(type2 && c[i][j]==(c[i - 3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3)){
	k=i-3;
	l=j+3;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di3;
	*dGy+=dj3;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-3]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2)){
	k=i-3;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di3;
	*dGy+=dj2;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-2]][S2[j+3]];
      if(type2 && c[i][j]==(c[i - 2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3)){
	k=i-2;
	l=j+3;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGx+=di2;
	*dGy+=dj3;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      //!!if(c[i][j]==(in[i - 2][j+2]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2+iext_s)){
      if(c[i][j]==(in[i - 3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di3+dj3+2*iext_s)){
	k=i;
	l=j;
	//!	*dGplex+=P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+iext_s;
	*dGplex+=P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*iext_s;
	
	//!*dGx+=di2;
	//!*dGy+=dj2;
	*dGx+=di3;
	*dGy+=dj3;
	st1[i-1] = '(';
	st2[j-1] = ')';
	//!	i=i-2;
	//!     j=j+2;
	i=i-3;
	j=j+3;
	state=2;
	traced=1;
	break;
      }
      if(c[i][j]==(bx[i - 2][j+1]+di2+dj1+bext)){
	k=i;
	l=j;
	st1[i-1] = '(';
	st2[j-1] = ')';
	*dGplex+=bext;
	*dGx+=di2;
	*dGy+=dj1;
	i=i-2;
	j=j+1;
	state=3;
	traced=1;
	break;
      }
      if(c[i][j]==(by[i - 1][j+2]+di1+dj2+bext)){
	k=i;
	l=j;
	*dGplex+=bext;
	*dGx+=di1;
	*dGy+=dj2;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=i-1;
	j=j+2;
	state=4;
	traced=1;
	break;
      }
      break;
    case 2:
      if(in[i][j]==(in[i - 1][j+1]+iext_s + di1 + dj1)){
	i--;
	j++;
	*dGplex=iext_s;
	*dGx+=di1;
	*dGy+=dj1;
	state=2;
	traced=1;
	break;
      }
      if(in[i][j]==(in[i - 1][j]+iext_ass + di1)){
	i=i-1;
	*dGplex+=iext_ass;
	*dGx+=di1;
	state=2;
	traced=1;
	break;
      }
      if(in[i][j]==(in[i][j+1]+iext_ass + dj1)){
	j++;
	state=2;
	*dGy+=dj1;
	*dGplex+=iext_ass;
	traced=1;
	break;
      }
      type2=pair[S2[j+1]][S1[i-1]];
      if(type2 && in[i][j]==(c[i - 1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s + di1 +dj1)){
	int temp; temp=k; k=i-1; i=temp;
	temp=l; l=j+1; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s;
	*dGx+=di1;
	*dGy+=dj1;
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    case 3:
      if(bx[i][j]==(bx[i - 1][j]+bext+di1)){
	i--;
	*dGplex+=bext;
	*dGx+=di1;
	state=3;
	traced=1;
	break;
      }
      type2=pair[S2[j]][S1[i-1]];
      if(type2 && bx[i][j]==(c[i - 1][j]+bopen+bext+(type2>2?P->TerminalAU:0)+di1)){
	int temp; temp=k; k=i-1; i=temp;
	temp=l; l=j; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=bopen+bext+(type2>2?P->TerminalAU:0);
	*dGx+=di1;
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    case 4:
      if(by[i][j]==(by[i][j+1] + bext +dj1)){
	j++;
	*dGplex+=bext;
	state=4;
	traced=1;
	break;
      }
      type2=pair[S2[j+1]][S1[i]];
      if(type2 && by[i][j]==(c[i][j+1]+bopen+bext+(type2>2?P->TerminalAU:0) + dj1)){
	int temp; temp=k; k=i; i=temp;
	temp=l; l=j+1; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	*dGplex+=bopen+bext+(type2>2?P->TerminalAU:0);
	*dGy+=dj1;
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    }
  }
  if (!traced) { 
    idiff=i_pos-(n3-10-i);
    jdiff=j_pos+j-11;
    E=c[i][j];
    if (i>1) {E -= P->dangle5[type][SS1[i-1]]; *dG+=P->dangle5[type][SS1[i-1]];*dGplex+=P->dangle5[type][SS1[i-1]];}
    if (j<n4){E -= P->dangle3[type][SS2[j+1]]; *dG+=P->dangle3[type][SS2[j+1]];*dGplex+=P->dangle3[type][SS2[j+1]];}
    if (type>2) {E -= P->TerminalAU; *dG+=P->TerminalAU;*dGplex+=P->TerminalAU;}
    if (E != P->DuplexInit+access_s1[1][idiff]+access_s2[1][jdiff]) {
      nrerror("backtrack failed in second fold duplex");
    }
    else{
      *dG+=P->DuplexInit;
      *dGplex+=P->DuplexInit;
      *dGx+=access_s1[1][idiff];
      *dGy+=access_s2[1][jdiff];
      st1[i-1]='(';
      st2[j-1]=')';
    }
  }
  if (i>11)  i--;
  if (j<n4-10) j++;
  struc = (char *) space(i0-i+1+j-j0+1+2);
  for (k=MAX2(i,1); k<=i0; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j0; k<=j; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i-1,0)); 
  strcat(struc, "&"); 
  strcat(struc, st2+j0-1);  
  /* printf("%s %3d,%-3d : %3d,%-3d\n", struc, i,i0,j0,j);  */
  free(st1); free(st2);
  return struc;
}

duplexT** Lduplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int threshold, const int alignment_length, const int delta, const int fast, const int il_a, const int il_b, const int b_a, const int b_b) //deprecated
{
  
  int i, j;
  int bopen=b_b;
  int bext=b_a;
  int iopen=il_b;
  int iext_s=2*il_a;//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  //int temp;
  int min_j_colonne;
  int max=INF;
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1
  //int nsubopt=10;  //total number of subopt
  int *position; //contains the position of the hits with energy > E
  int *position_j;
  n1 = (int) strlen(s1);
  n2 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  position = (int *) space ((delta+n1+3+delta) * sizeof(int));
  position_j= (int *) space ((delta+n1+3+delta) * sizeof(int));
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
    update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  lc = (int **) space(sizeof(int *) * 4);
  lc[0] = (int *) space(sizeof(int) * (n2+5)); 
  lc[1] = (int *) space(sizeof(int) * (n2+5));
  lc[2] = (int *) space(sizeof(int) * (n2+5));
  lc[3] = (int *) space(sizeof(int) * (n2+5));
  
  /*   local int array initialisation-------------------------------------------*/
  lin = (int**) space (sizeof(int *) * 4);
  lin[0] = (int *) space(sizeof(int) * (n2+5));
  lin[1] = (int *) space(sizeof(int) * (n2+5));
  lin[2] = (int *) space(sizeof(int) * (n2+5));
  lin[3] = (int *) space(sizeof(int) * (n2+5));
  
  /* local bx array initialization--------------------------------------------*/ 
  lbx = (int**) space (sizeof(int *) * 4);
  lbx[0] = (int *) space(sizeof(int) * (n2+5));
  lbx[1] = (int *) space(sizeof(int) * (n2+5));
  lbx[2] = (int *) space(sizeof(int) * (n2+5));
  lbx[3] = (int *) space(sizeof(int) * (n2+5));
  
  /*  local by array initialization--------------------------------------------*/ 
  lby = (int**) space (sizeof(int *) * 4);
  lby[0] = (int *) space(sizeof(int) * (n2+5));
  lby[1] = (int *) space(sizeof(int) * (n2+5));
  lby[2] = (int *) space(sizeof(int) * (n2+5));
  lby[3] = (int *) space(sizeof(int) * (n2+5));
  
  int di1,di2,di3,di4; //contains accessibility penalty
  
  
  
  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(j=n2+4;j>=0;j--) {
    /*  --------------------------------------------------------local array*/
    lbx[0][j]=lbx[1][j]=lbx[2][j]=lbx[3][j]=INF;//no bulge before 1
    lin[0][j]=lin[1][j]=lin[2][j]=lin[3][j]=INF;//no in before  1
    lc[0][j] =lc[1][j] =lc[2][j] = lc[3][j]=INF; //no bulge and no in before n2
    lby[0][j]=lby[1][j]=lby[2][j]=lby[3][j]=INF;
  }
  /*--------------------------------------------------------local array*/
  
  
  //-------------------------------------------------------------matrix initialisierung
  i=10 /*target_dead*/; //start from 2 (	i=4) because no structure allowed to begin with a single base pair
  i_length= n1 - 9  /*- target_dead*/ ; 
  while(i < i_length) {
    int idx=i%4;
    int idx_1=(i-1)%4;
    int idx_2=(i-2)%4;
    int idx_3=(i-3)%4;
    di1 = access_s1[5][i]   - access_s1[4][i-1];	   
    di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;
    di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2;
    di4 = access_s1[5][i-3] - access_s1[4][i-4] + di3;
    di1=MIN2(di1,165);
    di2=MIN2(di2,330);
    di3=MIN2(di3,495);
    j=n2 - 9 /*- query_dead*/; //start from n2-1 because no structure allow to begin with a single base pair 
    while (--j > 9/*query_dead - 1*/) {
      //----------------------------------------------------------update lin lbx lby matrix
      int dj1,dj2,dj3,dj4;
      dj1 = access_s2[5][j+4] - access_s2[4][j+4];	
      dj2 = access_s2[5][j+5] - access_s2[4][j+5] + dj1;
      dj3 = access_s2[5][j+6] - access_s2[4][j+6] + dj2;
      dj4 = access_s2[5][j+7] - access_s2[4][j+7] + dj3;
      dj1=MIN2(dj1,165);
      dj2=MIN2(dj2,330);
      dj3=MIN2(dj3,495);
      int type2, type,temp;
      type  = pair[S1[i]][S2[j]];
      lc[idx][j]= type ? P->DuplexInit + access_s1[1][i] + access_s2[1][j] : INF;
      type2=pair[S2[j+1]][S1[i-1]];
      lin[idx][j]=MIN2(lc[idx_1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+di1+dj1+iopen+iext_s,lin[idx_1][j]+iext_ass + di1); 
      lin[idx][j]=MIN2(lin[idx][j],lin[idx][j+1]+iext_ass + dj1); //Iij=ci-1j+1+M+iopen+2iext
      lin[idx][j]=MIN2(lin[idx][j],lin[idx_1][j+1]+iext_s + di1 + dj1);
      type2=pair[S2[j]][S1[i-1]];
      lbx[idx][j]=MIN2(lbx[idx_1][j]+bext + di1, lc[idx_1][j]+bopen+bext+(type2>2?P->TerminalAU:0) + di1);
      type2=pair[S2[j+1]][S1[i]];
      lby[idx][j]=MIN2(lby[idx][j+1]+bext + dj1 , lc[idx][j+1]+bopen+bext+(type2>2?P->TerminalAU:0)+ dj1);
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;} 
      lc[idx][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+P->dangle3[type][SS2[j+1]];
      //------------------------------------------------------------------update c  matrix 
      type2=pair[S1[i-1]][S2[j+1]]; 
      lc[idx][j]=MIN2(lc[idx_1][j+1]+P->stack[type2][rtype[type]]+di1+dj1, lc[idx][j]); //0x0+1x1
      type2=pair[S1[i-1]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2,lc[idx][j]);//1x0 + 1x1
      type2=pair[S1[i-2]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1,lc[idx][j]);//0x1 +1x1
      type2=pair[S1[i-2]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2, lc[idx][j]); // 1x1 +1x1
      type2 = pair[S1[i-3]][S2[j+3]];
      lc[idx][j]=MIN2(lc[idx_3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3,lc[idx][j]);//2x2 + 1x1
      type2 = pair[S1[i-3]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2, lc[idx][j]); // 2x1 +1x1
      type2 = pair[S1[i-2]][S2[j+3]];
      lc[idx][j]=MIN2(lc[idx_2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3, lc[idx][j]);// 1x2 +1x1
      //internal->stack 
      lc[idx][j]=MIN2(lin[idx_3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di3+dj3+2*iext_s, lc[idx][j]); //
      //bulge -> stack
      lc[idx][j]=MIN2(lbx[idx_2][j+1]+di2+dj1+bext, lc[idx][j]);
      //min2=by[i][j+1];
      lc[idx][j]=MIN2(lby[idx_1][j+2]+di1+dj2+bext, lc[idx][j]);
      temp=min_colonne;
      min_colonne=MIN2(lc[idx][j]+(type>2?P->TerminalAU:0)+
               	       P->dangle3[rtype[type]][SS1[i+1]]+
               	       P->dangle5[rtype[type]][SS2[j-1]], 
               	       min_colonne);
      if(temp>min_colonne){
	min_j_colonne=j;
      }
      
      //---------------------------------------------------------------------end update
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    }
    position[i+delta]=min_colonne;min_colonne=INF;
    position_j[i+delta]=min_j_colonne;
    i++;
  }
  free(S1); free(S2); free(SS1); free(SS2);  
  if(max<threshold){
    find_max_XS(position, position_j, delta, threshold, alignment_length, s1, s2, access_s1, access_s2, fast,il_a, il_b,b_a, b_b);  
  }
  if(max < INF){
    plot_max_XS(max, max_pos, max_pos_j,alignment_length, s1, s2, access_s1, access_s2,fast, il_a, il_b, b_a, b_b);
  }
  for (i=1; i<=3; i++) {free(lc[i]);free(lin[i]);free(lbx[i]);free(lby[i]);}
  free(lc[0]);free(lin[0]);free(lbx[0]);free(lby[0]);
  free(lc);free(lin);free(lbx);free(lby);
  free(position);
  free(position_j);
  return NULL;
}

duplexT** FLduplexfold_XS(const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int threshold, const int alignment_length, const int delta, const int fast, const int il_a, const int il_b, const int b_a, const int b_b) 
{
  int i, j;
  int bopen=b_b;
  int bext=b_a;
  int iopen=il_b;
  int iext_s=2*il_a;//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  //int temp;
  int min_j_colonne;
  int max=INF;
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1
  //int nsubopt=10;  //total number of subopt
  int *position; //contains the position of the hits with energy > E
  int *position_j;
  n1 = (int) strlen(s1);
  n2 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  position = (int *) space ((delta+n1+3+delta) * sizeof(int));
  position_j= (int *) space ((delta+n1+3+delta) * sizeof(int));
  //contains the penalty for dj, so that we dont have to compute it for 
  //each i
  int **DJ;
  DJ=(int **) space(3*sizeof(int*));
  DJ[0]=(int *) space(n2*sizeof(int));
  DJ[1]=(int *) space(n2*sizeof(int));
  DJ[2]=(int *) space(n2*sizeof(int));

  //instead of having 4 2-dim arrays we use a unique 1-dim array, in order to see if we can achieve some speed improvemet.
  int *SA;
  SA=(int *) space(sizeof(int)*4*4*(n2+5));
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
    update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  int di1,di2,di3,di4; //contains accessibility penalty
  
  
  
  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(j=n2+4;j>=0;j--) {
    /*  --------------------------------------------------------local array*/
    SA[(j*16)   ]=SA[(j*16)+1   ]=SA[(j*16)+2   ]=SA[(j*16)+3   ]=INF;
    SA[(j*16)+4 ]=SA[(j*16)+1+4 ]=SA[(j*16)+2+4 ]=SA[(j*16)+3+4 ]=INF;
    SA[(j*16)+8 ]=SA[(j*16)+1+8 ]=SA[(j*16)+2+8 ]=SA[(j*16)+3+8 ]=INF;
    SA[(j*16)+12]=SA[(j*16)+1+12]=SA[(j*16)+2+12]=SA[(j*16)+3+12]=INF;
  }
  //define index to simplify access to the SA single array approach;
  
  /*--------------------------------------------------------local array*/
  j=n2-9;
  while(--j>9){
    DJ[0][j] = MIN2(access_s2[5][j+4] - access_s2[4][j+4],165);	
    DJ[1][j] = MIN2(access_s2[5][j+5] - access_s2[4][j+5] + DJ[0][j],330);
    DJ[2][j] = MIN2(access_s2[5][j+6] - access_s2[4][j+6] + DJ[1][j],495);
  }
  //-------------------------------------------------------------matrix initialisierung
  i=10 /*target_dead*/; //start from 2 (	i=4) because no structure allowed to begin with a single base pair
  i_length= n1 - 9  /*- target_dead*/ ; 
  while(i < i_length) {
    int idx=i%4;
    int idx_1=(i-1)%4;
    int idx_2=(i-2)%4;
    int idx_3=(i-3)%4;
    di1 = access_s1[5][i]   - access_s1[4][i-1];	   
    di2 = access_s1[5][i-1] - access_s1[4][i-2] + di1;
    di3 = access_s1[5][i-2] - access_s1[4][i-3] + di2;
    di4 = access_s1[5][i-3] - access_s1[4][i-4] + di3;
    di1=MIN2(di1,165);
    di2=MIN2(di2,330);
    di3=MIN2(di3,495);

    j=n2 - 9 /*- query_dead*/; //start from n2-1 because no structure allow to begin with a single base pair 
    while (--j > 9/*query_dead - 1*/) {
      //----------------------------------------------------------update lin lbx lby matrix
      int dj1,dj2,dj3,dj4;
      dj1=DJ[0][j];
      dj2=DJ[1][j];
      dj3=DJ[2][j];
      
      int type2, type,temp;
      type  = pair[S1[i]][S2[j]];
      SA[LCI(idx,j,n2)] = type ? P->DuplexInit + access_s1[1][i] + access_s2[1][j] : INF;
      type2=pair[S2[j+1]][S1[i-1]];
      SA[LINI(idx,j,n2)]=MIN2(SA[LCI(idx_1,j+1,n2)]+P->mismatchI[type2][SS2[j]][SS1[i]]+di1+dj1+iopen+iext_s,SA[LINI(idx_1,j,n2)]+iext_ass + di1);
      SA[LINI(idx,j,n2)]=MIN2(SA[LINI(idx,j,n2)],SA[LINI(idx,j+1,n2)]+iext_ass + dj1);
      SA[LINI(idx,j,n2)]=MIN2(SA[LINI(idx,j,n2)],SA[LINI(idx_1,j+1,n2)]+iext_s + di1 + dj1);
      type2=pair[S2[j]][S1[i-1]];
      SA[LBXI(idx,j,n2)]=MIN2(SA[LBXI(idx_1,j,n2)]+bext + di1, SA[LCI(idx_1,j,n2)]+bopen+bext+(type2>2?P->TerminalAU:0) + di1);
      type2=pair[S2[j+1]][S1[i]];
      SA[LBYI(idx,j,n2)]=MIN2(SA[LBYI(idx,j+1,n2)]+bext + dj1 , SA[LCI(idx,j+1,n2)]+bopen+bext+(type2>2?P->TerminalAU:0)+ dj1);

      //--------------------------------------------------------------- end update recursion
      if(!type){continue;} 
      SA[LCI(idx,j,n2)]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+P->dangle3[type][SS2[j+1]];
      //------------------------------------------------------------------update c  matrix 
      if(type2=pair[S1[i-1]][S2[j+1]]) 
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_1,j+1,n2)]+P->stack[type2][rtype[type]]+di1+dj1, SA[LCI(idx,j,n2)]);
     if(type2=pair[S1[i-1]][S2[j+2]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_1,j+2,n2)]+P->stack[type2][rtype[type]]+P->bulge[1]+di1+dj2, SA[LCI(idx,j,n2)]);
      if(type2=pair[S1[i-2]][S2[j+1]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_2,j+1,n2)]+P->stack[type2][rtype[type]]+P->bulge[1]+di2+dj1, SA[LCI(idx,j,n2)]);
      if(type2=pair[S1[i-2]][S2[j+2]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_2,j+2,n2)]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+di2+dj2, SA[LCI(idx,j,n2)]);
      if(type2 = pair[S1[i-3]][S2[j+3]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_3,j+3,n2)]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di3+dj3, SA[LCI(idx,j,n2)]);
      if(type2 = pair[S1[i-3]][S2[j+2]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_3,j+2,n2)]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+di3+dj2, SA[LCI(idx,j,n2)]);
      if(type2 = pair[S1[i-2]][S2[j+3]])
      SA[LCI(idx,j,n2)]=MIN2(SA[LCI(idx_2,j+3,n2)]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+di2+dj3, SA[LCI(idx,j,n2)]);
      SA[LCI(idx,j,n2)]=MIN2(SA[LINI(idx_3,j+3,n2)]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+di3+dj3+2*iext_s,SA[LCI(idx,j,n2)]);
      SA[LCI(idx,j,n2)]=MIN2(SA[LBXI(idx_2,j+1,n2)]+di2+dj1+bext,SA[LCI(idx,j,n2)]);
      SA[LCI(idx,j,n2)]=MIN2(SA[LBYI(idx_1,j+2,n2)]+di1+dj2+bext,SA[LCI(idx,j,n2)]);
      temp=min_colonne;
      min_colonne=MIN2(SA[LCI(idx,j,n2)]+(type>2?P->TerminalAU:0)+
               	       P->dangle3[rtype[type]][SS1[i+1]]+
               	       P->dangle5[rtype[type]][SS2[j-1]], 
               	       min_colonne);
      
      if(temp>min_colonne){
	min_j_colonne=j;
      }
      
      //---------------------------------------------------------------------end update
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    }
    position[i+delta]=min_colonne;min_colonne=INF;
    position_j[i+delta]=min_j_colonne;
    i++;
  }
  free(S1); free(S2); free(SS1); free(SS2);free(SA);  
  if(max<threshold){
    find_max_XS(position, position_j, delta, threshold, alignment_length, s1, s2, access_s1, access_s2, fast,il_a, il_b,b_a, b_b);  
  }
  if(max<INF){
    plot_max_XS(max, max_pos, max_pos_j,alignment_length, s1, s2, access_s1, access_s2,fast, il_a, il_b, b_a, b_b); 
  }
  for (i=1; i<=3; i++) {
    free(DJ[i-1]);
  }
  free(DJ);
  free(position);
  free(position_j);
  return NULL;
}

PRIVATE void find_max_XS(const int *position, const int *position_j,const int delta, const int threshold, const int alignment_length, const char *s1, const char *s2, const int **access_s1, const int **access_s2, const int fast,const int il_a, const int il_b, const int b_a, const int b_b){
  
  int n_max;n_max=16;
  int pos=n1-9;
  if(fast==1){
    while(10 < pos--){
      printf("%d %d\n",pos,position[pos+delta]);
      int temp_min=0;
      if(position[pos+delta]<(threshold)){                        		
	int search_range;                                           	
	search_range=delta+1;                                       	
	while(--search_range){   
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;                                  	
	  }                                                         	
	}                                                           	
	pos-=temp_min;                                              	
	int max_pos_j;                                              	
	max_pos_j=position_j[pos+delta];
	int max;
	max=position[pos+delta];
	printf("target upper bound %d: query lower bound %d  (%5.2f) \n", pos-10, max_pos_j-10, ((double)max)/100);
	pos=MAX2(10,pos-delta);
      }
    }
  }
  else if(fast==2){
    pos=n1-9;
    while(10 < pos--){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min;
	int max_pos_j;
	max_pos_j=position_j[pos+delta];
	/* max_pos_j und pos entsprechen die realen position
 	   in der erweiterten sequenz. 
	   pos=1 -> position 1 in the sequence (and not 0 like in C)
	   max_pos_j -> position 1 in the sequence ( not 0 like in C)
	*/
	int begin_t=MAX2(11, pos-alignment_length+1);//10
	int end_t  =MIN2(n1-10, pos+1);
	int begin_q=MAX2(11, max_pos_j-1); //10
	int end_q  =MIN2(n2-10, max_pos_j+alignment_length-1);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2 + 20));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2 + 20));
	strcpy(s3,"NNNNNNNNNN");strcpy(s4,"NNNNNNNNNN");
	strncat(s3, (s1+begin_t-1),  end_t - begin_t +1);
	strncat(s4, (s2+begin_q-1) , end_q - begin_q +1);
	strcat(s3,"NNNNNNNNNN");strcat(s4,"NNNNNNNNNN");
	s3[end_t -begin_t +1 +20 ]='\0';
	s4[end_q -begin_q +1 +20]='\0';
	duplexT test;
	test = fduplexfold_XS(s3, s4, access_s1, access_s2, end_t, begin_q,threshold, il_a, il_b, b_a, b_b);
	if(test.energy * 100 < threshold){
	  int l1=strchr(test.structure, '&')-test.structure;
	  printf("FRA %s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f) [%5.2f = %5.2f + %5.2f + %5.2f]  %d \n", test.structure, 
		 begin_t-10+test.i-l1-10, 
		 begin_t-10+test.i-1-10, 
		 begin_q-10 + test.j-1-10 , 
		 (begin_q -11) + test.j + strlen(test.structure)-l1-2-10, 
		 test.ddG, test.energy, test.opening_backtrack_x, test.opening_backtrack_y,
		 test.energy_backtrack + test.dG1 + test.dG2, test.energy_backtrack , test.dG1 , test.dG2,
		 position[pos+delta]);
	  pos=MAX2(10,pos-delta);
	  free(s3);free(s4);
	  free(test.structure);
	}
	else{
	  free(s3);free(s4);
	}
      }
    }
  }
  else{
    pos=n1-9;
    while( pos-- > 10 ){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min; //position on i
	int max_pos_j;
	max_pos_j=position_j[pos+delta]; //position on j
	//int begin_t=MAX2(9, pos-alignment_length);
	//int end_t  =MIN2(n1-10, pos);
	//int begin_q=MAX2(9, max_pos_j-2);
	//int end_q  =MIN2(n2-10, max_pos_j+alignment_length-2);
	int begin_t=MAX2(9,pos-alignment_length);
	int end_t  =pos;
	int begin_q=max_pos_j-2;
	int end_q  =MIN2(n2-9,max_pos_j+alignment_length-2);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
	strncpy(s3, (s1+begin_t),  end_t - begin_t+1);
	strncpy(s4, (s2+begin_q) , end_q - begin_q+1 );
	s3[end_t -begin_t +1 ]='\0';
	s4[end_q -begin_q +1 ]='\0';
	duplexT test;
	test = duplexfold_XS(s3,s4,access_s1,access_s2,pos, max_pos_j,threshold);
	if(test.energy * 100 < threshold){
	  printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n", test.structure, 
		 test.tb,test.te,test.qb,test.qe, test.ddG, test.energy, test.dG1, test.dG2);
	  pos=MAX2(10,pos-delta);
	}
	free(s3);free(s4);
	free(test.structure);
      }
    }
  }
}

//PRIVATE int compare(const void *sub1, const void *sub2) {
//  int d;
//  if (((duplexT *) sub1)->ddG > ((duplexT *) sub2)->ddG)
//    return 1;
//  if (((duplexT *) sub1)->ddG < ((duplexT *) sub2)->ddG)
//    return -1;
//  d = ((duplexT *) sub1)->i - ((duplexT *) sub2)->i;
//  if (d!=0) return d;
//  return  ((duplexT *) sub1)->j - ((duplexT *) sub2)->j;
//}

PRIVATE void plot_max_XS(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int ** access_s1, const int ** access_s2, const int fast,const int il_a, const int il_b, const int b_a, const int b_b)
{
  if(fast==1){ 
    printf("target upper bound %d: query lower bound %d (%5.2f)\n", max_pos-3, max_pos_j, ((double)max)/100); 
  } 
  else if(fast==2){
    int begin_t=MAX2(11, max_pos-alignment_length+1);//10
    int end_t  =MIN2(n1-10, max_pos+1);
    int begin_q=MAX2(11, max_pos_j-1); //10
    int end_q  =MIN2(n2-10, max_pos_j+alignment_length-1);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2 + 20));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2 + 20));
    strcpy(s3,"NNNNNNNNNN");strcpy(s4,"NNNNNNNNNN");
    strncat(s3, (s1+begin_t-1),  end_t - begin_t +1);
    strncat(s4, (s2+begin_q-1) , end_q - begin_q +1);
    strcat(s3,"NNNNNNNNNN");strcat(s4,"NNNNNNNNNN");
    s3[end_t -begin_t +1 +20 ]='\0';
    s4[end_q -begin_q +1 +20]='\0';
    duplexT test;
    test = fduplexfold_XS(s3, s4, access_s1, access_s2, end_t, begin_q, INF, il_a, il_b, b_a, b_b);
    int l1=strchr(test.structure, '&')-test.structure;
    printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f) [%5.2f = %5.2f + %5.2f + %5.2f] \n", test.structure, 
	   begin_t-10+test.i-l1-10, 
	   begin_t-10+test.i-1-10, 
	   begin_q-10 + test.j-1-10 , 
	   (begin_q -11) + test.j + strlen(test.structure)-l1-2-10, 
	   test.ddG, test.energy, test.opening_backtrack_x, test.opening_backtrack_y,
	   test.energy_backtrack + test.dG1 + test.dG2, test.energy_backtrack , test.dG1 , test.dG2,
	   test);
    free(s3);free(s4);
    free(test.structure);
  }
  else{ 
    int begin_t=MAX2(9,max_pos-alignment_length);
    int end_t  =max_pos;
    int begin_q=max_pos_j-2;
    int end_q  =MIN2(n2-9,max_pos_j+alignment_length-2);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
    strncpy(s3, (s1+begin_t),  end_t - begin_t+1);
    strncpy(s4, (s2+begin_q) , end_q - begin_q+1);
    s3[end_t -begin_t +1 ]='\0';
    s4[end_q -begin_q +1 ]='\0';
    duplexT test;
    test = duplexfold_XS(s3,s4,access_s1,access_s2,max_pos, max_pos_j,INF);
    printf("%s %3d,%-3d : %3d,%-3d (%5.2f = %5.2f + %5.2f + %5.2f)\n", test.structure, 
	   test.tb,test.te,test.qb,test.qe, test.ddG, test.energy, test.dG1, test.dG2);
    free(s3);free(s4);free(test.structure);
  }
}


/*---------------------------------------------------------duplexfold----------------------------------------------------------------------------------*/


PRIVATE duplexT duplexfold(const char *s1, const char *s2, const int extension_cost) {
  int i, j, l1, Emin=INF, i_min=0, j_min=0;
  char *struc;
  duplexT mfe;
  
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);
 
  if ((!P) || (fabs(P->temperature - temperature)>1e-6)) {
    update_fold_params();  P = scale_parameters();
    make_pair_matrix();
  }
  
  c = (int **) space(sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++) c[i] = (int *) space(sizeof(int) * (n4+1));  
  encode_seqs(s1, s2);
  for (i=1; i<=n3; i++) {
    for (j=n4; j>0; j--) {
      int type, type2, E, k,l;
      type = pair[S1[i]][S2[j]];
      c[i][j] = type ? P->DuplexInit +2 * extension_cost: INF;
      if (!type) continue;
      if (i>1)  c[i][j] += P->dangle5[type][SS1[i-1]]+ extension_cost;
      if (j<n4) c[i][j] += P->dangle3[type][SS2[j+1]]+ extension_cost;
      if (type>2) c[i][j] += P->TerminalAU;
      for (k=i-1; k>0 && k>i-MAXLOOP-2; k--) {
	for (l=j+1; l<=n4; l++) {
	  if (i-k+l-j-2>MAXLOOP) break;
	  type2 = pair[S1[k]][S2[l]];
	  if (!type2) continue;
	  E = LoopEnergy(i-k-1, l-j-1, type2, rtype[type],
			 SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1])+(i-k+l-j)*extension_cost;
	  c[i][j] = MIN2(c[i][j], c[k][l]+E);
	}
      }
      E = c[i][j]; 
      if (i<n3) E += P->dangle3[rtype[type]][SS1[i+1]]+extension_cost;
      if (j>1)  E += P->dangle5[rtype[type]][SS2[j-1]]+extension_cost;
      if (type>2) E += P->TerminalAU;
      if (E<Emin) {
	Emin=E; i_min=i; j_min=j;
      } 
    }
  }
  struc = backtrack(i_min, j_min, extension_cost);
  if (i_min<n3) i_min++;
  if (j_min>1 ) j_min--;
  l1 = strchr(struc, '&')-struc;
  int size;
  size=strlen(struc)-1;
  Emin-= size * (extension_cost);
  mfe.i = i_min;
  mfe.j = j_min;
  mfe.energy = (double) Emin/100.;
  mfe.structure = struc;
  if (!delay_free) {
    for (i=0; i<=n3; i++) free(c[i]);
    free(c);
    free(S1); free(S2); free(SS1); free(SS2);
  }
  return mfe;
}

PRIVATE char *backtrack(int i, int j, const int extension_cost) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0;
  char *st1, *st2, *struc;
  
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  
  i0=MIN2(i+1,n3); j0=MAX2(j-1,1);

  while (i>0 && j<=n4) {
    E = c[i][j]; traced=0;
    st1[i-1] = '(';
    st2[j-1] = ')'; 
    type = pair[S1[i]][S2[j]];
    if (!type) nrerror("backtrack failed in fold duplex");
    for (k=i-1; k>0 && k>i-MAXLOOP-2; k--) {
      for (l=j+1; l<=n4; l++) {
	int LE;
	if (i-k+l-j-2>MAXLOOP) break;
	type2 = pair[S1[k]][S2[l]];
	if (!type2) continue;
	LE = LoopEnergy(i-k-1, l-j-1, type2, rtype[type],
			SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1])+(i-k+l-j)*extension_cost;
	if (E == c[k][l]+LE) {
	  traced=1; 
	  i=k; j=l;
	  break;
	}
      }
      if (traced) break;
    }
    if (!traced) { 
      if (i>1) E -= P->dangle5[type][SS1[i-1]]+extension_cost;
      if (j<n4) E -= P->dangle3[type][SS2[j+1]]+extension_cost;
      if (type>2) E -= P->TerminalAU;
      if (E != P->DuplexInit+2*extension_cost) {
	nrerror("backtrack failed in fold duplex");
      } else break;
    }
  }
  if (i>1)  i--;
  if (j<n4) j++;
  
  struc = (char *) space(i0-i+1+j-j0+1+2);
  for (k=MAX2(i,1); k<=i0; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j0; k<=j; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i-1,0)); 
  strcat(struc, "&"); 
  strcat(struc, st2+j0-1);  
  /* printf("%s %3d,%-3d : %3d,%-3d\n", struc, i,i0,j0,j);  */
  free(st1); free(st2);
  return struc;
}

PRIVATE duplexT fduplexfold(const char *s1, const char *s2, const int extension_cost,const int il_a, const int il_b, const int b_a, const int b_b) {
  int i, j, Emin, i_min, j_min,l1;
  duplexT mfe;
  char *struc;
  int bopen=b_b;
  int bext=b_a+extension_cost;
  int iopen=il_b;
  int iext_s=2*(il_a+extension_cost);//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a+extension_cost;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  int temp;
  int min_j_colonne;
  int max=INF;
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1

  int *position; //contains the position of the hits with energy > E
  int *position_j;
  n3 = (int) strlen(s1);
  n4 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
  update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  c  = (int**)  space(sizeof(int *) * (n3+1));
  in = (int**) space (sizeof(int *) * (n3+1));
  bx = (int**) space (sizeof(int *) * (n3+1));
  by = (int**) space (sizeof(int *) * (n3+1));
  for (i=0; i<=n3; i++){
    c[i]  = (int *) space(sizeof(int) * (n4+1));  
    in[i] = (int *) space(sizeof(int) * (n4+1));  
    bx[i] = (int *) space(sizeof(int) * (n4+1));  
    by[i] = (int *) space(sizeof(int) * (n4+1));  
  }
  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(i=8; i<11;i++){
    for(j=n4-7;j>=8;j--) {
      in[i][j]=INF;//no in before  1
      c[i][j] =INF; //no bulge and no in before n2
      bx[i][j]=INF;//no bulge before 1
      by[i][j]=INF;
 
      in[n3-i+1][j]=INF;//no in before  1
      c[n3-i+1][j] =INF; //no bulge and no in before n2
      bx[n3-i+1][j]=INF;//no bulge before 1
      by[n3-i+1][j]=INF;
    }
  }
  for(j=8; j < 11; j++) {
    for(i=10; i<n3-9; i++){
      in[i][j]=INF;//no in before  1
      c[i][j] =INF; //no bulge and no in before n2
      bx[i][j]=INF;//no bulge before 1
      by[i][j]=INF;
      
      in[i][n4-j+1]=INF;//no in before  1
      c[i][n4-j+1] =INF; //no bulge and no in before n2
      bx[i][n4-j+1]=INF;//no bulge before 1
      by[i][n4-j+1]=INF;
    }
  }
  /*--------------------------------------------------------local array*/
  
  
  //-------------------------------------------------------------matrix initialisierung
  i=11;
  i_length=n3-9;
  while(i < i_length) {
    j=n4-9;
    min_colonne=INF;
    while (10 < --j) {
      int type, type2; 
      type = pair[S1[i]][S2[j]];
      //----------------LOCAL
      //initialize lc to 0;
      c[i][j]=type ? P->DuplexInit + 2*extension_cost : INF; //to avoid that previous value influence result should actually not be erforderlich
      //----------------------------------------------------------update lin bx by matrix
      type2=pair[S2[j+1]][S1[i-1]];
      in[i][j]=MIN2(c[i - 1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s, in[i - 1][j]+iext_ass); //Iij=ci-1j+1+M+iopen+2iext
      in[i][j]=MIN2(in[i][j],in[i][j+1]+iext_ass);
      in[i][j]=MIN2(in[i][j],in[i - 1][j+1]+iext_s);
      type2=pair[S2[j]][S1[i-1]];
      bx[i][j]=MIN2(bx[i - 1][j]+bext, c[i - 1][j]+bopen+bext+(type2>2?P->TerminalAU:0));
      type2=pair[S2[j+1]][S1[i]];
      by[i][j]=MIN2(by[i][j+1]+bext, c[i][j+1]+bopen+bext+(type2>2?P->TerminalAU:0));
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;}
      c[i][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+extension_cost+P->dangle3[type][SS2[j+1]]+extension_cost;
      //type > 2 -> no GC or CG pair
      //------------------------------------------------------------------update c  matrix 
      type2=pair[S1[i-1]][S2[j+1]];
      c[i][j]=MIN2(c[i - 1][j+1]+P->stack[type2][rtype[type]]+2*extension_cost, c[i][j]);
      type2=pair[S1[i-1]][S2[j+2]];
      c[i][j]=MIN2(c[i - 1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,c[i][j]);
      type2=pair[S1[i-2]][S2[j+1]];
      c[i][j]=MIN2(c[i - 2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,c[i][j]);
      type2=pair[S1[i-2]][S2[j+2]];
      c[i][j]=MIN2(c[i - 2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+4*extension_cost, c[i][j]);
      type2 = pair[S1[i-3]][S2[j+3]];
      c[i][j]=MIN2(c[i - 3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+6*extension_cost,c[i][j]);
      type2 = pair[S1[i-3]][S2[j+2]];
      c[i][j]=MIN2(c[i - 3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+5*extension_cost, c[i][j]);
      type2 = pair[S1[i-2]][S2[j+3]];
      c[i][j]=MIN2(c[i - 2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+5*extension_cost, c[i][j]);
      //internal->stack 
      c[i][j]=MIN2(in[i - 3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*extension_cost+2*iext_s, c[i][j]); //
      //bulge -> stack
      c[i][j]=MIN2(bx[i - 2][j+1]+2*extension_cost+bext, c[i][j]);
      //min2=by[i][j+1];
      c[i][j]=MIN2(by[i - 1][j+2]+2*extension_cost+bext, c[i][j]);
      temp=min_colonne;
      min_colonne=MIN2(c[i][j]+(type>2?P->TerminalAU:0)+P->dangle3[rtype[type]][SS1[i+1]]+extension_cost+P->dangle5[rtype[type]][SS2[j-1]]+extension_cost, min_colonne);
      if(temp>min_colonne){
	min_j_colonne=j;
      }
      //---------------------------------------------------------------------end update
    }
    if(max>=min_colonne){
      max=min_colonne;
      max_pos=i;
      max_pos_j=min_j_colonne;
    }
    i++;
  }
  Emin=max;
  i_min=max_pos;
  j_min=max_pos_j;
  int dGe;
  dGe=0;
  struc = fbacktrack(i_min, j_min, extension_cost, il_a, il_b, b_a, b_b,&dGe);
  if (i_min<n3-10) i_min++;
  if (j_min>11 ) j_min--;
  l1 = strchr(struc, '&')-struc;
  int size;
  size=strlen(struc)-1;
  Emin-= size * (extension_cost);
  mfe.i = i_min;
  mfe.j = j_min;
  mfe.energy = (double) Emin/100.;
  mfe.energy_backtrack = (double) dGe/100.;
  mfe.structure = struc;
  free(S1); free(S2); free(SS1); free(SS2);  
  for (i=0; i<=n3; i++) {
    free(c[i]);
    free(in[i]);
    free(bx[i]);
    free(by[i]);
  }
  free(c);free(in);free(bx);free(by);
  return mfe;
}


PRIVATE char *fbacktrack(int i, int j, const int extension_cost,const int il_a, const int il_b, const int b_a, const int b_b, int *dG) {
  /* backtrack structure going backwards from i, and forwards from j 
     return structure in bracket notation with & as separator */
  int k, l, type, type2, E, traced, i0, j0;
  char *st1, *st2, *struc;
  int bopen=b_b;
  int bext=b_a+extension_cost;
  int iopen=il_b;
  int iext_s=2*(il_a+extension_cost);//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a+extension_cost;//iext_ass assymetric extension of interior loop, either on i or on j side.
  st1 = (char *) space(sizeof(char)*(n3+1));
  st2 = (char *) space(sizeof(char)*(n4+1));
  i0=MIN2(i+1,n3-10); j0=MAX2(j-1,11);
  int state;
  state=1; //we start backtracking from a a pair , i.e. c-matrix
  /* state 1 -> base pair, c
     state 2 -> interior loop, in
     state 3 -> bx loop, bx
     state 4 -> by loop, by
  */
  traced=1;
  k=i;l=j;
  type=pair[S1[i]][S2[j]];
  *dG+=(type>2?P->TerminalAU:0)+P->dangle3[rtype[type]][SS1[i+1]]+P->dangle5[rtype[type]][SS2[j-1]];
  while (i>10 && j<=n4-9 && traced) {
    traced=0;
    switch(state){
    case 1:
      type = pair[S1[i]][S2[j]];
      if(!type) nrerror("backtrack failed in fold duplex");
      type2=pair[S1[i-1]][S2[j+1]];
      if(type2 && c[i][j]== (c[i - 1][j+1]+P->stack[type2][rtype[type]]+2*extension_cost)){
	k=i-1;
	l=j+1;
	(*dG)+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-1]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost)){
	k=i-1;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-2]][S2[j+1]];
      if(type2 && c[i][j]==(c[i - 2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost)){
	k=i-2;
	l=j+1;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2=pair[S1[i-2]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+4*extension_cost)){
	k=i-2;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-3]][S2[j+3]];
      if(type2 && c[i][j]==(c[i - 3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+6*extension_cost)){
	k=i-3;
	l=j+3;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-3]][S2[j+2]];
      if(type2 && c[i][j]==(c[i - 3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+5*extension_cost)){
	k=i-3;
	l=j+2;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      type2 = pair[S1[i-2]][S2[j+3]];
      if(type2 && c[i][j]==(c[i - 2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+5*extension_cost)){
	k=i-2;
	l=j+3;
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
      if(c[i][j]==(in[i - 3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*extension_cost+2*iext_s)){
	k=i;
	l=j;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=i-3;
	j=j+3;
	state=2;
	traced=1;
	break;
      }
      if(c[i][j]==(bx[i - 2][j+1]+2*extension_cost+bext)){
	k=i;
	l=j;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=i-2;
	j=j+1;
	state=3;
	traced=1;
	break;
      }
      if(c[i][j]==(by[i - 1][j+2]+2*extension_cost+bext)){
	k=i;
	l=j;
	st1[i-1] = '(';
	st2[j-1] = ')';
	i=i-1;
	j=j+2;
	state=4;
	traced=1;
	break;
      }
      break;
    case 2:
      if(in[i][j]==(in[i - 1][j+1]+iext_s)){
	i--;
	j++;
	state=2;
	traced=1;
	break;
      }
      if(in[i][j]==(in[i - 1][j]+iext_ass)){
	i=i-1;
	state=2;
	traced=1;
	break;
      }
      if(in[i][j]==(in[i][j+1]+iext_ass)){
	j++;
	state=2;
	traced=1;
	break;
      }
      type2=pair[S2[j+1]][S1[i-1]];
      if(type2 && in[i][j]==(c[i - 1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s)){
	int temp; temp=k; k=i-1; i=temp;
	temp=l; l=j+1; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    case 3:
      if(bx[i][j]==(bx[i - 1][j]+bext)){
	i--;
	state=3;
	traced=1;
	break;
      }
      type2=pair[S2[j]][S1[i-1]];
      if(type2 && bx[i][j]==(c[i - 1][j]+bopen+bext+(type2>2?P->TerminalAU:0))){
	int temp; temp=k; k=i-1; i=temp;
	temp=l; l=j; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    case 4:
      if(by[i][j]==(by[i][j+1] + bext)){
	j++;
	
	state=4;
	traced=1;
	break;
      }
      type2=pair[S2[j+1]][S1[i]];
      if(type2 && by[i][j]==(c[i][j+1]+bopen+bext+(type2>2?P->TerminalAU:0))){
	int temp; temp=k; k=i; i=temp;
	temp=l; l=j+1; j=temp;
	type=pair[S1[i]][S2[j]];
	*dG+=LoopEnergy(i-k-1, l-j-1, type2, rtype[type],SS1[k+1], SS2[l-1], SS1[i-1], SS2[j+1]);
	i=k;
	j=l;
	state=1;
	traced=1;
	break;
      }
    }
  }
  if (!traced) { 
    E=c[i][j];
    if (i>1) {E -= P->dangle5[type][SS1[i-1]]+extension_cost; *dG+=P->dangle5[type][SS1[i-1]];}
    if (j<n4){E -= P->dangle3[type][SS2[j+1]]+extension_cost; *dG+=P->dangle3[type][SS2[j+1]];}
    if (type>2) {E -= P->TerminalAU; *dG+=P->TerminalAU;}
    if (E != P->DuplexInit+2*extension_cost) {
      nrerror("backtrack failed in second fold duplex");
    }
    else{
      *dG+=P->DuplexInit;
      st1[i-1]='(';
      st2[j-1]=')';
    }
  }
  if (i>11)  i--;
  if (j<n4-10) j++;
  struc = (char *) space(i0-i+1+j-j0+1+2);
  for (k=MAX2(i,1); k<=i0; k++) if (!st1[k-1]) st1[k-1] = '.';
  for (k=j0; k<=j; k++) if (!st2[k-1]) st2[k-1] = '.';
  strcpy(struc, st1+MAX2(i-1,0)); 
  strcat(struc, "&"); 
  strcat(struc, st2+j0-1);  
  /* printf("%s %3d,%-3d : %3d,%-3d\n", struc, i,i0,j0,j);  */
  free(st1); free(st2);
  return struc;
}

duplexT ** Lduplexfold(const char *s1, const char *s2, const int threshold, const int extension_cost, const int alignment_length, const int delta, const int fast, const int il_a, const int il_b, const int b_a, const int b_b)
{
  int i, j;
  int bopen=b_b;
  int bext=b_a+extension_cost;
  int iopen=il_b;
  int iext_s=2*(il_a+extension_cost);//iext_s 2 nt nucleotide extension of interior loop, on i and j side
  int iext_ass=50+il_a+extension_cost;//iext_ass assymetric extension of interior loop, either on i or on j side.
  int min_colonne=INF; //enthaelt das maximum einer kolonne
  int i_length;
  int max_pos;//get position of the best hit
  int max_pos_j;
  int temp;
  int min_j_colonne;
  int max=INF;
  //FOLLOWING NEXT 4 LINE DEFINES AN ARRAY CONTAINING POSITION OF THE SUBOPT IN S1

  int *position; //contains the position of the hits with energy > E
  int *position_j;
  n1 = (int) strlen(s1);
  n2 = (int) strlen(s2);
  //delta_check is the minimal distance allowed for two hits to be accepted
  //if both hits are closer, reject the smaller ( in term of position)  hits 
  position = (int *) space ((delta+n1+3+delta) * sizeof(int));
  position_j= (int *) space ((delta+n1+3+delta) * sizeof(int));
  //i want to implement a function that, given a position in a long sequence and a small sequence,
  //duplexfold them at this position and report the result at the command line//
  //for this i first need to rewrite backtrack in order to remove the printf functio
  //END OF DEFINITION FOR NEEDED SUBOPT DATA 
  
  if ((!P) || (fabs(P->temperature - temperature)>1e-6))
  update_dfold_params();
  /*local c array initialization---------------------------------------------*/
  lc = (int **) space(sizeof(int *) * 4);
  lc[0] = (int *) space(sizeof(int) * (n2+5)); 
  lc[1] = (int *) space(sizeof(int) * (n2+5));
  lc[2] = (int *) space(sizeof(int) * (n2+5));
  lc[3] = (int *) space(sizeof(int) * (n2+5));

/*   local int array initialisation-------------------------------------------*/
  lin = (int**) space (sizeof(int *) * 4);
  lin[0] = (int *) space(sizeof(int) * (n2+5));
  lin[1] = (int *) space(sizeof(int) * (n2+5));
  lin[2] = (int *) space(sizeof(int) * (n2+5));
  lin[3] = (int *) space(sizeof(int) * (n2+5));

/*  local bx array initialization--------------------------------------------*/ 
  lbx = (int**) space (sizeof(int *) * 4);
  lbx[0] = (int *) space(sizeof(int) * (n2+5));
  lbx[1] = (int *) space(sizeof(int) * (n2+5));
  lbx[2] = (int *) space(sizeof(int) * (n2+5));
  lbx[3] = (int *) space(sizeof(int) * (n2+5));

/*   local by array initialization--------------------------------------------*/ 
  lby = (int**) space (sizeof(int *) * 4);
  lby[0] = (int *) space(sizeof(int) * (n2+5));
  lby[1] = (int *) space(sizeof(int) * (n2+5));
  lby[2] = (int *) space(sizeof(int) * (n2+5));
  lby[3] = (int *) space(sizeof(int) * (n2+5));

  /*-------------------------------------------------------------------------*/
  /*end of array initialisation----------------------------------*/
  /*maybe int *** would be better*/
  encode_seqs(s1,s2);
  //------------------------------------------matrix initialisierung
  for(j=n2;j>=0;j--) {
   /*  --------------------------------------------------------local array*/
    lbx[0][j]=lbx[1][j]=lbx[2][j]=lbx[3][j]=INF;//no bulge before 1
    lin[0][j]=lin[1][j]=lin[2][j]=lin[3][j]=INF;//no in before  1
    lc[0][j] =lc[1][j] =lc[2][j] = lc[3][j]=INF; //no bulge and no in before n2
    lby[0][j]=lby[1][j]=lby[2][j]=lby[3][j]=INF;
    

  }
  /*--------------------------------------------------------local array*/

  
  //-------------------------------------------------------------matrix initialisierung
  i=10;
  i_length= n1 - 9 ;
  while(i < i_length) {
    int idx=i%4;
    int idx_1=(i-1)%4;
    int idx_2=(i-2)%4;
    int idx_3=(i-3)%4;
    j=n2-9;
    while (9 < --j) {
      int type, type2; 
      type = pair[S1[i]][S2[j]];
      //----------------LOCAL
      //initialize lc to 0;
      lc[idx][j]=type ? P->DuplexInit + 2*extension_cost : INF; //to avoid that previous value influence result should actually not be erforderlich
      //----------------------------------------------------------update lin lbx lby matrix
      type2=pair[S2[j+1]][S1[i-1]];
      lin[idx][j]=MIN2(lc[idx_1][j+1]+P->mismatchI[type2][SS2[j]][SS1[i]]+iopen+iext_s, lin[idx_1][j]+iext_ass); //Iij=ci-1j+1+M+iopen+2iext
      lin[idx][j]=MIN2(lin[idx][j],lin[idx][j+1]+iext_ass);
      lin[idx][j]=MIN2(lin[idx][j],lin[idx_1][j+1]+iext_s);
      type2=pair[S2[j]][S1[i-1]];
      lbx[idx][j]=MIN2(lbx[idx_1][j]+bext, lc[idx_1][j]+bopen+bext+(type2>2?P->TerminalAU:0));
      type2=pair[S2[j+1]][S1[i]];
      lby[idx][j]=MIN2(lby[idx][j+1]+bext, lc[idx][j+1]+bopen+bext+(type2>2?P->TerminalAU:0));
      //--------------------------------------------------------------- end update recursion
      if(!type){continue;}
      lc[idx][j]+=(type>2?P->TerminalAU:0)+P->dangle5[type][SS1[i-1]]+extension_cost+P->dangle3[type][SS2[j+1]]+extension_cost;
      //type > 2 -> no GC or CG pair
      //------------------------------------------------------------------update c  matrix 
      type2=pair[S1[i-1]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_1][j+1]+P->stack[type2][rtype[type]]+2*extension_cost, lc[idx][j]);
      type2=pair[S1[i-1]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_1][j+2]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,lc[idx][j]);
      type2=pair[S1[i-2]][S2[j+1]];
      lc[idx][j]=MIN2(lc[idx_2][j+1]+P->stack[type2][rtype[type]]+P->bulge[1]+3*extension_cost,lc[idx][j]);
      type2=pair[S1[i-2]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_2][j+2]+P->int11[type2][rtype[type]][SS1[i-1]][SS2[j+1]]+4*extension_cost, lc[idx][j]);
      type2 = pair[S1[i-3]][S2[j+3]];
      lc[idx][j]=MIN2(lc[idx_3][j+3]+P->int22[type2][rtype[type]][SS1[i-2]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+6*extension_cost,lc[idx][j]);
      type2 = pair[S1[i-3]][S2[j+2]];
      lc[idx][j]=MIN2(lc[idx_3][j+2]+P->int21[rtype[type]][type2][SS2[j+1]][SS1[i-2]][SS1[i-1]]+5*extension_cost, lc[idx][j]);
      type2 = pair[S1[i-2]][S2[j+3]];
      lc[idx][j]=MIN2(lc[idx_2][j+3]+P->int21[type2][rtype[type]][SS1[i-1]][SS2[j+1]][SS2[j+2]]+5*extension_cost, lc[idx][j]);
      //internal->stack 
      lc[idx][j]=MIN2(lin[idx_3][j+3]+P->mismatchI[rtype[type]][SS1[i-1]][SS2[j+1]]+2*extension_cost+2*iext_s, lc[idx][j]); //
      //bulge -> stack
      lc[idx][j]=MIN2(lbx[idx_2][j+1]+2*extension_cost+bext, lc[idx][j]);
      //min2=by[i][j+1];
      lc[idx][j]=MIN2(lby[idx_1][j+2]+2*extension_cost+bext, lc[idx][j]);

      temp=min_colonne;
      min_colonne=MIN2(lc[idx][j]+(type>2?P->TerminalAU:0)+P->dangle3[rtype[type]][SS1[i+1]]+extension_cost+P->dangle5[rtype[type]][SS2[j-1]]+extension_cost, min_colonne);
      if(temp>min_colonne){
	      min_j_colonne=j;
      }
      //---------------------------------------------------------------------end update
      }
   if(max>=min_colonne){
    	max=min_colonne;
    	max_pos=i;
	max_pos_j=min_j_colonne;
    }


    position[i+delta]=min_colonne;min_colonne=INF;
    position_j[i+delta]=min_j_colonne;
    i++;
 }

  free(S1); free(S2); free(SS1); free(SS2);  
  if(max<threshold){
    find_max(position, position_j, delta, threshold, alignment_length, s1, s2, extension_cost, fast, il_a, il_b, b_a, b_b);
  }
  plot_max(max, max_pos, max_pos_j,alignment_length, s1, s2, extension_cost,fast, il_a, il_b, b_a, b_b);
  for (i=1; i<=3; i++) {free(lc[i]);free(lin[i]);free(lbx[i]);free(lby[i]);}
  free(lc[0]);free(lin[0]);free(lbx[0]);free(lby[0]);
  free(lc);free(lin);free(lbx);free(lby);
  free(position);
  free(position_j);
  return NULL;
}


PRIVATE void find_max(const int *position, const int *position_j,const int delta, const int threshold, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast,const int il_a, const int il_b, const int b_a, const int b_b){
  int pos=n1-9;
  if(fast==1){
    while(10 < pos--){
      int temp_min=0;                                               		
      if(position[pos+delta]<(threshold)){                          		
	int search_range;                                           		
	search_range=delta+1;                                       		
	while(--search_range){                                      		
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){		
	    temp_min=search_range;                                  		
	  }                                                         		
	}                                                           		
	pos-=temp_min;                                              		
	int max_pos_j;                                              		
	max_pos_j=position_j[pos+delta];
	int max;
	max=position[pos+delta];
	printf("target upper bound %d: query lower bound %d  (%5.2f) \n", pos-10, max_pos_j-10, ((double)max)/100);
	pos=MAX2(10,pos-alignment_length);
      }
    }
  }
  else if(fast==2){
    pos=n1-9;
    while(10 < pos--){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min;
	int max_pos_j;
	max_pos_j=position_j[pos+delta];
	/* max_pos_j und pos entsprechen die realen position
	   in der erweiterten sequenz. 
	   pos=1 -> position 1 in the sequence (and not 0 like in C)
	   max_pos_j -> position 1 in the sequence ( not 0 like in C)
	*/
	int begin_t=MAX2(11, pos-alignment_length+1);//10
	int end_t  =MIN2(n1-10, pos+1);
	int begin_q=MAX2(11, max_pos_j-1); //10
	int end_q  =MIN2(n2-10, max_pos_j+alignment_length-1);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2 + 20));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2 + 20));
	strcpy(s3,"NNNNNNNNNN");strcpy(s4,"NNNNNNNNNN");
	strncat(s3, (s1+begin_t-1),  end_t - begin_t +1);
	strncat(s4, (s2+begin_q-1) , end_q - begin_q +1);
	strcat(s3,"NNNNNNNNNN");strcat(s4,"NNNNNNNNNN");
	s3[end_t -begin_t +1 +20 ]='\0';
	s4[end_q -begin_q +1 +20]='\0';
	duplexT test;
	test = fduplexfold(s3, s4, extension_cost,il_a, il_b, b_a, b_b);
	if(test.energy * 100 < threshold){
	  int l1=strchr(test.structure, '&')-test.structure;
	  printf("%s %3d,%-3d : %3d,%-3d (%5.2f) [%5.2f]\n", test.structure, 
		 begin_t-10+test.i-l1-10, 
		 begin_t-10+test.i-1-10, 
		 begin_q-10 + test.j-1-10 , 
		 (begin_q -11) + test.j + strlen(test.structure)-l1-2-10, 
		 test.energy,test.energy_backtrack);
	  pos=MAX2(10,pos-delta);
	}
	free(s3);free(s4);
	free(test.structure);
      }
    }
  }
  else{
    pos=n1-9;
    while(10 < pos--){
      int temp_min=0;
      if(position[pos+delta]<(threshold)){
	int search_range;
	search_range=delta+1;
	while(--search_range){ 
	  if(position[pos+delta-search_range]<=position[pos+delta-temp_min]){
	    temp_min=search_range;
	  }
	}
	pos-=temp_min;
	int max_pos_j;
	max_pos_j=position_j[pos+delta];
	/* max_pos_j und pos entsprechen die realen position
	   in der erweiterten sequenz. 
	   pos=1 -> position 1 in the sequence (and not 0 like in C)
	   max_pos_j -> position 1 in the sequence ( not 0 like in C)
	*/
	int begin_t=MAX2(11, pos-alignment_length+1);//10
	int end_t  =MIN2(n1-10, pos+1);
	int begin_q=MAX2(11, max_pos_j-1); //10
	int end_q  =MIN2(n2-10, max_pos_j+alignment_length-2);
	char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
	char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
	strncpy(s3, (s1+begin_t-1),  end_t - begin_t +1);
	strncpy(s4, (s2+begin_q-1) , end_q - begin_q +1);
	s3[end_t -begin_t +1 ]='\0';
	s4[end_q -begin_q +1 ]='\0';
	duplexT test;
	test = duplexfold(s3, s4, extension_cost);
	if(test.energy * 100 < threshold){
	  int l1=strchr(test.structure, '&')-test.structure;
	  printf("%s %3d,%-3d : %3d,%-3d (%5.2f)\n", test.structure, 
		 begin_t-10+test.i-l1, 
		 begin_t-10+test.i-1, 
		 begin_q-10 + test.j-1 , 
		 (begin_q -11) + test.j + strlen(test.structure)-l1-2, 
		 test.energy);
	  pos=MAX2(10,pos-delta);
	}
	free(s3);free(s4);
	free(test.structure);
      }
    }
  }
}
PRIVATE void plot_max(const int max, const int max_pos, const int max_pos_j, const int alignment_length, const char *s1, const char *s2, const int extension_cost, const int fast,const int il_a, const int il_b, const int b_a, const int b_b)
{
  if(fast==1){
    printf("target upper bound %d: query lower bound %d (%5.2f)\n", max_pos-10, max_pos_j-10, ((double)max)/100);
  }
  else if(fast==2){
    int begin_t=MAX2(11, max_pos-alignment_length+1);//10
    int end_t  =MIN2(n1-10, max_pos+1);
    int begin_q=MAX2(11, max_pos_j-1); //10
    int end_q  =MIN2(n2-10, max_pos_j+alignment_length-1);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2 + 20));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2 + 20));
    strcpy(s3,"NNNNNNNNNN");strcpy(s4,"NNNNNNNNNN");
    strncat(s3, (s1+begin_t-1),  end_t - begin_t +1);
    strncat(s4, (s2+begin_q-1) , end_q - begin_q +1);
    strcat(s3,"NNNNNNNNNN");strcat(s4,"NNNNNNNNNN");
    s3[end_t -begin_t +1 +20 ]='\0';
    s4[end_q -begin_q +1 +20]='\0';
    duplexT test;
    test = fduplexfold(s3, s4, extension_cost,il_a, il_b, b_a, b_b);
    int l1=strchr(test.structure, '&')-test.structure;
    printf("%s %3d,%-3d : %3d,%-3d (%5.2f) [%5.2f]", test.structure, 
	   begin_t-10+test.i-l1-10, 
	   begin_t-10+test.i-1-10, 
	   begin_q-10 + test.j-1-10 , 
	   (begin_q -11) + test.j + strlen(test.structure)-l1-2-10, 
	   test.energy, test.energy_backtrack);
    free(s3);free(s4);free(test.structure);
  }
  else{
    duplexT test;
    int begin_t=MAX2(11, max_pos-alignment_length+1);
    int end_t  =MIN2(n1-10, max_pos+1);
    int begin_q=MAX2(11, max_pos_j-1);
    int end_q  =MIN2(n2-10, max_pos_j+alignment_length-1);
    char *s3 = (char*) space(sizeof(char)*(end_t - begin_t +2));
    char *s4 = (char*) space(sizeof(char)*(end_q - begin_q +2));
    strncpy(s3, (s1+begin_t-1),  end_t - begin_t + 1);
    strncpy(s4, (s2+begin_q-1) , end_q - begin_q +1 );
    s3[end_t -begin_t +1 ]='\0';
    s4[end_q -begin_q +1 ]='\0';    
    test = duplexfold(s3, s4, extension_cost);
    int l1=strchr(test.structure, '&')-test.structure;
    printf("%s %3d,%-3d : %3d,%-3d (%5.2f)", test.structure, 
	   begin_t-10+test.i-l1, 
	   begin_t-10+test.i-1, 
	   begin_q-10 +test.j-1 , 
	   (begin_q -11) + test.j + strlen(test.structure)-l1-2, 
	   test.energy);
    free(s3);free(s4);free(test.structure);
  }
}


PRIVATE void update_dfold_params(void)
{
  P = scale_parameters();
  make_pair_matrix();
}

PRIVATE void encode_seqs(const char *s1, const char *s2) {
  unsigned int i,l;

  l = strlen(s1);
  S1 = encode_seq(s1);
  SS1= (short *) space(sizeof(short)*(l+1));
  /* SS1 exists only for the special X K and I bases and energy_set!=0 */
  
  for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
    SS1[i] = alias[S1[i]];   /* for mismatches of nostandard bases */
  }

  l = strlen(s2);
  S2 = encode_seq(s2);
  SS2= (short *) space(sizeof(short)*(l+1));
  /* SS2 exists only for the special X K and I bases and energy_set!=0 */
  
  for (i=1; i<=l; i++) { /* make numerical encoding of sequence */
    SS2[i] = alias[S2[i]];   /* for mismatches of nostandard bases */
  }
}

PRIVATE short * encode_seq(const char *sequence) {
  unsigned int i,l;
  short *S;
  l = strlen(sequence);
  S = (short *) space(sizeof(short)*(l+2));
  S[0] = (short) l;

  /* make numerical encoding of sequence */
  for (i=1; i<=l; i++)
    S[i]= (short) encode_char(toupper(sequence[i-1]));

  /* for circular folding add first base at position n+1 */
  S[l+1] = S[1];

  return S;
}

int arraySize(duplexT** array)
{
  int site_count=0;
  while(array[site_count]!=NULL){
    site_count++;
  }
  return site_count;
}

void freeDuplexT(duplexT** array)
{
  int size=arraySize(array);
  while(--size){
    free(array[size]->structure);
    free(array[size]);
  }
  free(array[0]->structure);
  free(array);
}



